SystemOrganization addCategory: #FunctionalTalk!
SystemOrganization addCategory: #'FunctionalTalk-Tests'!


TestCase subclass: #FunctionalTalkTest
	instanceVariableNames: 'printPref'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FunctionalTalk-Tests'!

Object subclass: #Lambda
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FunctionalTalk'!

MessageSend subclass: #LambdaMessageSend
	instanceVariableNames: 'compiledForm slots dependents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FunctionalTalk'!

LambdaMessageSend subclass: #BoundLambdaMessageSend
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FunctionalTalk'!

Object subclass: #LambdaSlot
	instanceVariableNames: 'id default description type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FunctionalTalk'!

ProtoObject subclass: #LambdaWrapper
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FunctionalTalk'!

FunctionalTalkTest class
	instanceVariableNames: ''!

Lambda class
	instanceVariableNames: ''!

LambdaMessageSend class
	instanceVariableNames: ''!

BoundLambdaMessageSend class
	instanceVariableNames: ''!

LambdaSlot class
	instanceVariableNames: ''!

LambdaWrapper class
	instanceVariableNames: ''!


!BoundLambdaMessageSend commentStamp: 'spfa 1/18/2007 20:49' prior: 0!
a LambdaMessageSend without any mechanism for automatic expansion.

while autoexpansion is a quite useful feature when building lambda expressions, it can also cause rather hard to catch errors during a reduction process. 

so in some very specific places we replace a LambdaMessageSend with a safer equivalent BoundLambdaMessageSend.!

!FunctionalTalkTest commentStamp: '<historical>' prior: 0!
Unit tests for the package FunctionalTalk.

See the tests classified as "documentation" for an overview of the package.!

!Lambda commentStamp: 'spfa 1/19/2007 19:44' prior: 0!
this class simply provides syntactic sugar for lambda expressions.

	| f |

	"1) compact syntax for AtomicLambdaMessageSend creation"
	f _ Lambda x + (Lambda y * 5) sqrt. 		"printIt"

	"2) compact syntax for 'section' (Haskell vocabulary)"
	f <~~ {Lambda . 1}							"printIt"!

!LambdaMessageSend commentStamp: '<historical>' prior: 0!
a MessageSend suitable for functional programming:
its receiver, selector and arguments can themselves be LambdaMessageSends.

see the 'documentation' test cases in FunctionalTalkTest for a comprehensive overview of this topic 

!

!LambdaSlot commentStamp: 'spfa 1/19/2007 19:44' prior: 0!
a slot in a lambda construct. 

see LambdaMessageSend for details!

!LambdaWrapper commentStamp: 'spfa 5/15/2006 22:00' prior: 0!
catch as many DNUs as possible to form a LambdaMessageSend with receiver inst var object

see Object>>#quoted!
]style[(109)f1!

!BoundLambdaMessageSend methodsFor: 'private' stamp: 'spfa 1/18/2007 14:39'!
adaptToNumber: rcvr andSend: sel

	self error! !

!LambdaMessageSend methodsFor: 'easy arithmetic' stamp: 'spfa 1/18/2007 14:02'!
adaptToNumber: rcvr andSend: sel

	"allows things like	(1 + Lambda x)"

	| lform |

	lform _ rcvr lambda: sel.

	^  (lform substitute: {self}) ofVariables: self boundSlots! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 1/17/2007 20:24'!
adaptToWorld: aWorld

	"should we adapt somehow ?"! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 1/19/2007 19:23'!
allSlotIds

	^ self slots collect: [:aslot | aslot id].! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 12/11/2006 13:46'!
argumentsFromList: args

	args size > self boundSlots size 
		ifTrue: [self error: 'wrong number of arguments'].

	^ (self boundSlots first: args size) with: args 
			collect: [:bs :arg | bs id -> arg]

! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 1/19/2007 16:49'!
arguments: anObject

	anObject isLambda ifTrue: [arguments _ anObject lambdaClone]
		ifFalse: [arguments _ anObject collect: 
			[:arg | arg isLambda ifTrue: [arg _ arg lambdaClone] ifFalse: [arg]]].
	! !

!LambdaMessageSend methodsFor: 'evaluating' stamp: 'spfa 5/10/2006 15:36'!
argument: anObject

	super arguments: {anObject}! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 1/27/2007 11:57'!
arity

	self freeSlots ifNotEmpty: [^ nil].
	^ self boundVariables size! !

!LambdaWrapper class methodsFor: 'as yet unclassified' stamp: 'spfa 5/15/2006 21:53'!
around: anObject

	^ self new object: anObject! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 2/3/2007 21:37'!
asBlock

	| bslots dict blockCode objects |

	dict _ Dictionary new.

	blockCode _ (String streamContents: [:str | 
			str nextPut: $[.
			(bslots _ self boundSlots) do: [:arg |
				str nextPut: $:.
				arg lambdaStoreOn: str.
				str space].
			bslots size isZero ifFalse: [str nextPut: $|; space].
			self lambdaPrintOn: str dict: dict boundSlots: self boundSlots.
			str nextPut: $]]).

	dict size isZero ifTrue: [^ Compiler evaluate: blockCode].

	objects _ dict associations asArray.

	^ (Compiler evaluate: (String streamContents: [:str | 
			str nextPut: $[.
			(objects collect: [:o | o key]) do: [:arg |
				str nextPut: $:.
				str nextPutAll: arg.
				str space].
			str nextPut: $|; space.
			str nextPutAll: blockCode; nextPut: $]
		])) valueWithArguments: (objects collect: [:o | o value]) 

			
! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 2/8/2007 19:53'!
asLambdaMessageSend

	^ self! !

!MessageSend methodsFor: '*FunctionalTalk' stamp: 'spfa 2/8/2007 19:53'!
asLambdaMessageSend

	^ LambdaMessageSend 
		receiver: receiver selector: selector arguments: arguments! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 2/3/2007 20:28'!
boundSlots

	^ slots ifNil: [slots _ self getSlots reject: [:s | self hasInnerBindingFor: s]]! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 12/10/2006 15:25'!
boundSlots

	^ #()! !

!LambdaMessageSend methodsFor: 'evaluating' stamp: 'spfa 2/8/2007 13:56'!
boundSlotsWith: someslots

	^ self boundSlots, (someslots reject: [:s | self boundSlots identityIncludes: s])
! !

!LambdaSlot methodsFor: 'evaluating' stamp: 'spfa 12/5/2006 20:41'!
boundSlotsWith: slots

	^ (slots identityIncludes: self)
		ifTrue: [slots]
		ifFalse: [{self}, slots]
! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 2/8/2007 13:25'!
boundVariables

	^ self boundSlots collect: [:s | s id]! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 2/8/2007 14:16'!
compile

	"a LambdaMessageSend is not mutable, so if it has already been compiled there is no need to do it again"
	
	compiledForm ifNil: [compiledForm _ self asBlock].
		! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 1/19/2007 21:53'!
compileAs: aSelector in: aClass classified: heading

	aClass class compile: 
		(String streamContents: [:code | 
				self boundSlots with: aSelector keywords do: [:arg :word |
					code nextPutAll: word.
					code space.
					arg lambdaStoreOn: code.
					code space].
				code cr; cr; tab; nextPutAll: '^ '.
				self lambdaStoreOn: code.])
		classified: heading.

		! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 2/8/2007 14:14'!
compiled

	compiledForm ifNil: [self compile].
	^ compiledForm
! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 1/19/2007 21:53'!
compileIn: aClass classified: heading

	self compileAs: (String streamContents: [:sel | 	
							self boundSlots do: [:slot |
								slot lambdaStoreOn: sel.
								sel nextPut: $:]]) 
	in: aClass classified: heading
		! !

!LambdaMessageSend methodsFor: 'composition' stamp: 'spfa 1/20/2007 13:56'!
composeWith: aMessageSend

	^ aMessageSend <= self

! !

!LambdaMessageSend methodsFor: 'printing' stamp: 'spfa 5/13/2006 20:55'!
constructor

	^ String streamContents: [:stream | self writeConstructorOn: stream]! !

!LambdaSlot methodsFor: 'printing' stamp: 'spfa 1/19/2007 22:03'!
declareOn: stream

	self lambdaStoreOn: stream .
	stream nextPutAll: ' _ Lambda '.
	self lambdaStoreOn: stream .
	stream nextPut: $.; space




! !

!LambdaMessageSend methodsFor: 'copying' stamp: 'spfa 12/10/2006 17:31'!
deepCopy

	self eraseCompiledForm.
	slots _ nil.
	^super deepCopy

! !

!LambdaMessageSend methodsFor: 'parameters' stamp: 'spfa 1/19/2007 11:09'!
default

	^ (self boundSlots collect: [:slot | slot default]) ==> self! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 5/16/2006 13:19'!
default

	^ default! !

!LambdaMessageSend methodsFor: 'parameters' stamp: 'spfa 12/10/2006 17:54'!
defaultForm

	^ (self boundSlots select: [:slot | slot default ~~ Lambda] 
			thenCollect: [:slot | slot id -> slot default]) @@>> self

! !

!LambdaMessageSend methodsFor: 'parameters' stamp: 'spfa 2/4/2007 21:03'!
defaultFor: aSymbol

	^ (self boundSlots detect: [:bslot | aSymbol == bslot id]) default
! !

!LambdaMessageSend methodsFor: 'atomic access' stamp: 'spfa 2/2/2007 20:44'!
default: anObject

	self isAtomic ifFalse: [self error: 'not an atomic lambda'].

	receiver default: anObject! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 5/16/2006 13:19'!
default: anObject

	default _ anObject! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 5/16/2006 13:20'!
description

	^ description ! !

!LambdaMessageSend methodsFor: 'atomic access' stamp: 'spfa 2/2/2007 20:44'!
description: anObject

	self isAtomic ifFalse: [self error: 'not an atomic lambda'].

	receiver description: anObject! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 5/16/2006 13:19'!
description: aString

	description _ aString! !

!BoundLambdaMessageSend methodsFor: 'private' stamp: 'spfa 1/20/2007 21:17'!
doesNotUnderstand: aMessage 
	 "Handle the fact that there was an attempt to send the given message to the receiver but the receiver does not understand this message (typically sent from the machine when a message is sent to the receiver and no method is defined for that selector)."
	"Testing: (3 activeProcess)"

	MessageNotUnderstood new 
		message: aMessage;
		receiver: self;
		signal.
	^ aMessage sentTo: self.! !

!Lambda class methodsFor: 'as yet unclassified' stamp: 'spfa 2/2/2007 20:44'!
doesNotUnderstand: aMessage

	aMessage selector numArgs isZero
		ifFalse: [^ super doesNotUnderstand: aMessage].

	^ LambdaMessageSend 
		receiver: (LambdaSlot new id: aMessage selector)
		selector: nil
		arguments: #()! !

!LambdaMessageSend methodsFor: 'evaluating' stamp: 'spfa 2/9/2007 09:46'!
doesNotUnderstand: aMessage

	| argsSlots |

	argsSlots _ Array streamContents: [:str |
					(aMessage arguments select: [:arg | arg isLambda])
						do: [:arg | str nextPutAll: 
										(arg boundSlots reject: [:s | 
											str contents identityIncludes: s])]].

	^ (self class
		receiver: (self isWrappedReceiver ifFalse: [self] ifTrue: [self receiver])
		selector: aMessage selector
		arguments: aMessage arguments) 
	ofVariables: (self boundSlotsWith: argsSlots)! !

!LambdaSlot class methodsFor: 'API' stamp: 'spfa 5/9/2006 22:39'!
doesNotUnderstand: aMessage

	^ self selector: aMessage selector arguments: aMessage arguments! !

!LambdaWrapper methodsFor: 'as yet unclassified' stamp: 'spfa 2/9/2007 15:46'!
doesNotUnderstand: aMessage

	object isMessageSend ifTrue:
		[^ object asLambdaMessageSend doesNotUnderstand: aMessage].

	^ object lambda doesNotUnderstand: aMessage! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 12/6/2006 13:36'!
ensureSlotsArityFromIds
		
	self slots do: [:s1 | 
		self slots do: [:s2 | 
			((s1 == s2) not and: [s1 id == s2 id]) 
				ifTrue: [s1 becomeForward: s2]]].! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 1/16/2007 14:17'!
ensureUnambigousSlotIds
		
	| s |

	s _ self slots.

	s do: [:s1 | 
		s do: [:s2 | 
			((s1 == s2) not and: [s1 id == s2 id]) 
				ifTrue: [s1 getNewId]]].! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 5/12/2006 14:27'!
eraseCompiledForm

	compiledForm isBlock ifTrue: [compiledForm _ nil]
! !

!StringHolder methodsFor: '*FunctionalTalk' stamp: 'spfa 1/27/2007 10:47'!
exploreLambdaReferences

	(MessageSend listReferencesTo: self selectedMessageName) explore! !

!ParagraphEditor methodsFor: '*FunctionalTalk' stamp: 'spfa 1/27/2007 11:05'!
exploreLambdaReferencesToIt

	| sel o |

	o _ (sel _ self selection asString) first isLowercase
		ifTrue: [sel asSymbol]
		ifFalse: [Smalltalk at: sel asSymbol].
	
	(MessageSend listReferencesTo: o) explore
! !

!PluggableTextMorph methodsFor: '*FunctionalTalk' stamp: 'spfa 1/27/2007 11:02'!
exploreLambdaReferencesToIt

	textMorph editor exploreLambdaReferencesToIt! !

!MessageSend methodsFor: '*FunctionalTalk' stamp: 'spfa 1/27/2007 10:33'!
extends: aMessageSend

	^ receiver == aMessageSend 
		or: [selector == aMessageSend]
		or: [arguments == aMessageSend]
		or: [arguments notNil
				and: [arguments anySatisfy: [:arg | arg == aMessageSend]]]! !

!LambdaMessageSend methodsFor: 'evaluating' stamp: 'spfa 2/8/2007 22:56'!
forcedValueRecursive: flag

	| rec class val |

	"the following lines inhibit the expression autoexpansion mechanisms"
	rec _ receiver.
	(rec isKindOf: LambdaMessageSend) 
		ifTrue: [class _ rec class.
				rec _ receiver as: BoundLambdaMessageSend].
	(rec isNumber and: [arguments size = 1 and: [arguments first isLambda]])
		ifTrue: [^ self].

	"now we can safely reduce the expression"
	val _ flag ifTrue: 
		[
			rec lambdaValue
				perform: selector lambdaValue
				withArguments: (arguments collect: [:each | each lambdaValue])
		]
		ifFalse: [rec perform: selector withArguments: arguments].

	"back to standard LambdaMessageSend"
	(val isKindOf: BoundLambdaMessageSend) 
		ifTrue: [val _ val as: class].

	^ val! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 1/17/2007 19:10'!
freeInnerSlots: anArray

	receiver isLambda ifTrue: [receiver freeSlots: anArray].
	selector isLambda ifTrue: [selector freeSlots: anArray].
	arguments isLambda ifTrue: [arguments freeSlots: anArray]
		ifFalse: [arguments do: 
					[:arg | arg isLambda ifTrue: [arg freeSlots: anArray]]].
! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 1/19/2007 19:09'!
freeSlotIds

	^ self freeSlots collect: [:fslot | fslot id].! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 1/19/2007 16:26'!
freeSlots

	^ self unboundSlots reject: [:s | self hasBindingFor: s]! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 1/18/2007 19:13'!
freeSlots: anArray

	self slots: (self boundSlots reject: [:s | anArray identityIncludes: s]).
	self freeInnerSlots: anArray

! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 1/17/2007 19:12'!
freeSlots: anArray

! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 12/6/2006 13:56'!
getNewId

	id ifNil: [^ self id: #'1'].

	self id: (id endsWithDigit ifFalse: [id, '1']
				ifTrue: [	| n |
					id asInteger ifNil: [id _ id, '0'].												
					n _ id reversed initialIntegerOrNil asString reversed.
					(id allButLast: n size), (n asInteger + 1) asString]) asSymbol
! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 12/6/2006 14:44'!
getSlots

	^ self slots inject: #() into: [:uslots :s |
			(uslots identityIncludes: s) ifTrue: [uslots] ifFalse: [uslots, {s}]]! !

!LambdaMessageSend methodsFor: 'testing' stamp: 'spfa 2/3/2007 20:21'!
hasBindingFor: aLambdaSlot

	(self boundSlots identityIncludes: aLambdaSlot)
		ifTrue: [^ true].

	(receiver isLambda and: [receiver hasBindingFor: aLambdaSlot])
		ifTrue: [^ true].

	(selector isLambda and: [selector hasBindingFor: aLambdaSlot])
		ifTrue: [^ true].

	(arguments isLambda and: [arguments hasBindingFor: aLambdaSlot])
		ifTrue: [^ true].

	arguments isLambda ifFalse:
	[
		arguments do: [:arg |
			(arg isLambda and: [arg hasBindingFor: aLambdaSlot])
				ifTrue: [^ true]].	
	].

	^ false! !

!LambdaSlot methodsFor: 'testing' stamp: 'spfa 1/19/2007 16:28'!
hasBindingFor: aLambdaSlot

	^ false! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 5/12/2006 14:26'!
hasCompiledForm

	^ compiledForm notNil and: [compiledForm isBlock]
! !

!LambdaMessageSend methodsFor: 'testing' stamp: 'spfa 2/3/2007 20:21'!
hasInnerBindingFor: aLambdaSlot

	(receiver isLambda and: [receiver hasBindingFor: aLambdaSlot])
		ifTrue: [^ true].

	(selector isLambda and: [selector hasBindingFor: aLambdaSlot])
		ifTrue: [^ true].

	(arguments isLambda and: [arguments hasBindingFor: aLambdaSlot])
		ifTrue: [^ true].

	arguments isLambda ifFalse:
	[
		arguments do: [:arg |
			(arg isLambda and: [arg hasBindingFor: aLambdaSlot])
				ifTrue: [^ true]].	
	].

	^ false! !

!LambdaMessageSend methodsFor: 'atomic access' stamp: 'spfa 2/2/2007 20:44'!
id

	self isAtomic ifFalse: [self error: 'not an atomic lambda'].

	^ receiver id! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 5/9/2006 22:34'!
id

	^ id ! !

!LambdaSlot class methodsFor: 'instance creation' stamp: 'spfa 5/9/2006 14:45'!
id: anObject

	^ self new id: anObject! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 5/9/2006 14:43'!
id: anObject 

	id _ anObject! !

!False methodsFor: '*FunctionalTalk' stamp: 'spfa 2/9/2007 20:27'!
ifTrueDo: aLambda ifFalseDo: bLambda

	^ bLambda lambdaValue! !

!True methodsFor: '*FunctionalTalk' stamp: 'spfa 2/9/2007 20:28'!
ifTrueDo: aLambda ifFalseDo: bLambda

	^ aLambda lambdaValue! !

!LambdaSlot methodsFor: 'initialize-release' stamp: 'spfa 5/16/2006 13:26'!
initialize

	super initialize.
	default _ Lambda! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 1/18/2007 11:11'!
inlineSlots: associations 

	^ self substituteSlots: associations reduce: true inline: true! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 12/11/2006 14:02'!
inlineSlot: anAssociation 

	^ self inlineSlots: {anAssociation} ! !

!LambdaMessageSend methodsFor: 'testing' stamp: 'spfa 2/2/2007 20:44'!
isAtomic

	^ receiver isLambdaSlot
		 and: [selector isNil]
		 and: [arguments = #()]! !

!LambdaMessageSend methodsFor: 'testing' stamp: 'spfa 5/16/2006 09:17'!
isLambda

	^ receiver isLambda 
		or: [selector isLambda]
		or: [arguments isLambda]
		or: [arguments notNil
				and: [arguments anySatisfy: [:arg | arg isLambda]]]! !

!LambdaSlot methodsFor: 'testing' stamp: 'spfa 5/14/2006 14:14'!
isLambda

	^ true
! !

!Object methodsFor: '*FunctionalTalk' stamp: 'spfa 5/9/2006 10:23'!
isLambda

	^ false! !

!LambdaSlot methodsFor: 'testing' stamp: 'spfa 5/11/2006 14:02'!
isLambdaSlot

	^ true
! !

!Object methodsFor: '*FunctionalTalk' stamp: 'spfa 5/11/2006 14:03'!
isLambdaSlot

	^ false! !

!LambdaMessageSend methodsFor: 'testing' stamp: 'spfa 2/2/2007 20:44'!
isParametric

	^ self parameters notEmpty! !

!LambdaMessageSend methodsFor: 'testing' stamp: 'spfa 2/9/2007 09:46'!
isWrappedReceiver

	^ selector isNil and: [arguments = #()]! !

!BoundLambdaMessageSend methodsFor: 'private' stamp: 'spfa 1/18/2007 14:39'!
isZero

	self error! !

!LambdaMessageSend methodsFor: 'easy arithmetic' stamp: 'spfa 1/18/2007 14:02'!
isZero

	"allows things like	(1/(Lambda x + 1))"

	^ false
! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 2/9/2007 14:55'!
lambda

	^ super lambda privateReceiver: self! !

!LambdaSlot methodsFor: 'conversion' stamp: 'spfa 2/9/2007 15:01'!
lambda

	^ super lambda ofVariables: {}! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 2/3/2007 20:07'!
lambda

	^ LambdaMessageSend 
		receiver: self 
		selector: nil 
		arguments: #()
! !

!LambdaMessageSend methodsFor: 'copying' stamp: 'spfa 1/19/2007 16:49'!
lambdaClone

	^ self copy
		! !

!LambdaSlot methodsFor: 'conversion' stamp: 'spfa 1/19/2007 16:49'!
lambdaClone

	! !

!LambdaMessageSend methodsFor: 'copying' stamp: 'spfa 2/9/2007 21:56'!
lambdaCopy

	| dict |

	dict _ IdentityDictionary new.
	(slots, self slots) do: [:s | dict add: s -> s copy].

	^ self withNewSlots: dict
		! !

!LambdaSlot class methodsFor: 'API' stamp: 'spfa 12/6/2006 13:43'!
lambdaMessage

	^ (self id: #receiver) lambdaMessage! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 12/6/2006 13:44'!
lambdaMessage

	^ LambdaMessageSend 
		receiver: self 
		selector: (LambdaSlot id: #selector) 
		arguments: (LambdaSlot id: #arguments)
! !

!LambdaMessageSend methodsFor: 'printing' stamp: 'spfa 2/3/2007 21:42'!
lambdaPrintOn: stream dict: aDictionary boundSlots: anArray


	stream nextPut: $(.
	receiver lambdaRefOn: stream dict: aDictionary boundSlots: anArray.
	selector ifNotNil: [
     	stream space.
		(arguments isLambda not and: [arguments isEmpty]) ifTrue: [
				selector isLambda ifTrue:[
						stream nextPutAll: 'perform: '.
						selector lambdaRefOn: stream dict: aDictionary boundSlots: anArray]
					ifFalse: [stream nextPutAll: selector]]
			ifFalse: [
				(selector isLambda or: [arguments isLambda]) ifTrue:[
						stream nextPutAll: 'perform:'; space.
						selector lambdaRefOn: stream dict: aDictionary boundSlots: anArray.
						stream space; nextPutAll: 'withArguments:'; space.						
						arguments isLambda 
							ifTrue:[arguments lambdaRefOn: stream dict: aDictionary boundSlots: anArray]
							ifFalse: [
								stream nextPut: ${.
								arguments do: [:arg |
									arg lambdaRefOn: stream dict: aDictionary boundSlots: anArray.
									stream nextPutAll: ' . '].
								stream skip: -3; nextPut: $}]]
					ifFalse: [
						arguments with: selector keywords do: [:arg :word |
							stream nextPutAll: word.
							stream space.
							arg lambdaRefOn: stream dict: aDictionary boundSlots: anArray.
							stream space].
				stream skip: -1]]].	
	stream nextPut: $)! !

!LambdaMessageSend methodsFor: 'printing' stamp: 'spfa 2/3/2007 21:43'!
lambdaRefOn: aStream dict: aDictionary boundSlots: anArray

	self boundSlots isEmpty ifTrue: 
		[^ self lambdaPrintOn: aStream dict: aDictionary boundSlots: anArray].

	super lambdaRefOn: aStream dict: aDictionary boundSlots: anArray
! !

!LambdaSlot methodsFor: 'printing' stamp: 'spfa 2/3/2007 21:40'!
lambdaRefOn: stream dict: aDictionary boundSlots: anArray

	(anArray identityIncludes: self) ifTrue: [^ self lambdaStoreOn: stream].

	super lambdaRefOn: stream dict: aDictionary boundSlots: anArray




! !

!Object methodsFor: '*FunctionalTalk' stamp: 'spfa 2/3/2007 21:40'!
lambdaRefOn: aStream dict: aDictionary boundSlots: anArray

	| label |

	label _ 'o', (aDictionary size + 1) asString.
	aDictionary at: label put: self. 
	aStream nextPutAll: label! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 12/6/2006 13:30'!
lambdaSlots

	"slots which are not parameters"

	^ self boundSlots 
		select: [:slot | slot default == Lambda]
		
! !

!LambdaMessageSend methodsFor: 'printing' stamp: 'spfa 2/2/2007 20:44'!
lambdaStoreOn: stream 

	self isAtomic ifTrue: [^ receiver lambdaStoreOn: stream].

	stream nextPut: $(.
	self maybeWrap: receiver onStream: stream using: #lambdaStoreString.
	selector ifNotNil: [
     	stream space.
		(arguments isLambda not and: [arguments isEmpty]) ifTrue: [
				selector isLambda ifTrue:[
						stream nextPutAll: 'perform: '.
						selector lambdaStoreOn: stream]
					ifFalse: [stream nextPutAll: selector]]
			ifFalse: [
				(selector isLambda or: [arguments isLambda]) ifTrue:[
						stream nextPutAll: 'perform:'; space.
						selector lambdaStoreOn: stream.
						stream space; nextPutAll: 'withArguments:'; space.						
						arguments isLambda 
							ifTrue:[arguments lambdaStoreOn: stream]
							ifFalse: [
								stream nextPut: ${.
								arguments do: [:arg |
									arg lambdaStoreOn: stream.
									stream nextPutAll: ' . '].
								stream skip: -3; nextPut: $}]]
					ifFalse: [
						arguments with: selector keywords do: [:arg :word |
							stream nextPutAll: word.
							stream space.
							self maybeWrap: arg onStream: stream using: #lambdaStoreString.
							stream space].
				stream skip: -1]]].	
	stream nextPut: $)! !

!LambdaSlot methodsFor: 'printing' stamp: 'spfa 1/19/2007 21:52'!
lambdaStoreOn: stream

	"specific usage. see #declareOn: "

	 | varname |

	varname _ id asString.
	(varname allSatisfy: [:char | char isAlphaNumeric])
		ifFalse: [varname _ varname asIdentifier: false]
		ifTrue: [varname startsWithDigit ifTrue: [stream nextPutAll: 'arg']].

	stream nextPutAll: varname





! !

!Object methodsFor: '*FunctionalTalk' stamp: 'spfa 1/19/2007 21:50'!
lambdaStoreOn: aStream

	self storeOn: aStream! !

!Object methodsFor: '*FunctionalTalk' stamp: 'spfa 1/19/2007 22:05'!
lambdaStoreString

	^ String streamContents: [:s | self lambdaStoreOn: s]! !

!LambdaMessageSend methodsFor: 'printing' stamp: 'spfa 1/19/2007 22:08'!
lambdaStoreStringLimitedTo: limit

	| limitedString |
	limitedString _ String streamContents: [:s | self lambdaStoreOn: s] limitedTo: limit.
	limitedString size < limit ifTrue: [^ limitedString].
	^ limitedString , '...'
! !

!LambdaMessageSend methodsFor: 'evaluating' stamp: 'spfa 5/9/2006 11:46'!
lambdaValue

	^ self value! !

!Object methodsFor: '*FunctionalTalk' stamp: 'spfa 5/9/2006 11:46'!
lambdaValue

	^ self ! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 2/3/2007 20:47'!
lambda: aSymbolOrArray

	| slam |
	slam _ super lambda: aSymbolOrArray.
	slam receiver: (slam receiver ofVariables: self boundSlots).
	^ slam ofVariables: #()! !

!LambdaSlot class methodsFor: 'API' stamp: 'spfa 2/3/2007 21:17'!
lambda: anObjectOrArray

	"this is a bit obfuscated... could be simplified I guess
	we want the Object>>#lambda: to be applied to a LambdaSlot"

	| lms |

	lms _ (self id: #receiver) lambda: anObjectOrArray.
	^ lms ofVariables: lms slots! !

!LambdaSlot methodsFor: 'conversion' stamp: 'spfa 2/3/2007 21:26'!
lambda: aSymbolOrArray

	| slam |
	slam _ super lambda: aSymbolOrArray.
	^ slam ofVariables: (slam boundSlots reject: [:s | s == self])! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 12/11/2006 14:31'!
lambda: aSymbolOrArray

	aSymbolOrArray isSymbol 
	ifTrue:
	[
		| slots eslots |

		slots _ eslots _ (1 to: aSymbolOrArray numArgs) 
							collect: [:n | LambdaSlot id: n asString asSymbol].

		self isLambda ifTrue:
			[eslots _ self boundSlotsWith: slots].

		^ (LambdaMessageSend 
			receiver: self 
			selector: aSymbolOrArray 
			arguments: slots)
		ofVariables: eslots
	]
	ifFalse: "Array, then"
	[
		| slot eslots |

		slot _ LambdaSlot id: 'selector'.
		eslots _ self isLambda ifTrue: [self boundSlotsWith: {slot}]
					ifFalse: [{slot}].

		^ (LambdaMessageSend 
			receiver: self 
			selector: slot
			arguments: aSymbolOrArray) ofVariables: eslots
	]	

! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 12/6/2006 13:45'!
lambda: aSymbol slotIds: anArray

	^ LambdaMessageSend 
		receiver: self 
		selector: aSymbol 
		arguments: (anArray collect: [:n | LambdaSlot id: n asString asSymbol])

! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 12/6/2006 13:45'!
lambda: aSymbol slotId: bSymbol

	aSymbol numArgs ~= 1 ifTrue: [^ self error: 'more than one slot defined'].

	^ LambdaMessageSend 
		receiver: self 
		selector: aSymbol 
		arguments: {LambdaSlot id: bSymbol}

! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 2/9/2007 14:28'!
lambda: aSymbol withAll: anObject

	^ self lambda doesNotUnderstand: 
		(Message selector: aSymbol 
			arguments: (Array new: aSymbol numArgs withAll: anObject))

! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 2/9/2007 14:28'!
lambda: aSymbol withArguments: anArray

	^ self lambda doesNotUnderstand: 
		(Message selector: aSymbol arguments: anArray)


! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 2/9/2007 14:26'!
lambda: aSymbol with: anObject

	^ self lambda doesNotUnderstand: 
		(Message selector: aSymbol argument: anObject)! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 2/9/2007 14:27'!
lambda: aSymbol with: anObject with: bObject 

	^ self lambda doesNotUnderstand: 
		(Message selector: aSymbol arguments: {anObject . bObject})

! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 2/9/2007 14:27'!
lambda: aSymbol with: anObject with: bObject with: cObject

	^ self lambda doesNotUnderstand: 
		(Message selector: aSymbol arguments: {anObject . bObject . cObject})

! !

!MessageSend class methodsFor: '*FunctionalTalk' stamp: 'spfa 1/27/2007 10:33'!
listReferencesTo: anObject

	"very inefficient algorithm..."

	| all |

	all _ self allSubInstances select: [:ms | ms references: anObject].
	^ all select: [:ms |
			all noneSatisfy: [:ums | ums extends: ms]]! !

!LambdaMessageSend methodsFor: 'printing' stamp: 'spfa 1/19/2007 16:07'!
maybeWrap: arg onStream: stream using: aSelector

	| wflag astr |

	astr _ arg perform: aSelector.
	wflag _ (arg isNumber
				or: [arg isString
				or: [arg isBehavior
				or: [arg isLambda
				or: [astr first = $( and: [astr last = $)]]
				or: [(astr includesAnyOf: '+-*/;:.<>~@=') not]]]]									
				) not.
						
	wflag ifTrue: [stream nextPut: $(].
	stream nextPutAll: astr.
	wflag ifTrue: [stream nextPut: $)]! !

!LambdaMessageSend methodsFor: 'dependents' stamp: 'spfa 2/6/2007 23:20'!
myDependents

	"dependents are directly stored in an instance variable so that serialization works ok"

	^ dependents ifNil: [dependents _ DependentsArray new]! !

!LambdaMessageSend methodsFor: 'dependents' stamp: 'spfa 2/6/2007 20:54'!
myDependents: aDependentsArray 

	dependents _ aDependentsArray! !

!MessageSend class methodsFor: '*FunctionalTalk' stamp: 'spfa 1/27/2007 10:32'!
nonExtendedInstances

	"very inefficient algorithm..."

	| all |

	all _ self allInstances. 
	^ all select: [:ms | all noneSatisfy: [:ums | ums extends: ms]]! !

!MessageSend class methodsFor: '*FunctionalTalk' stamp: 'spfa 1/27/2007 10:32'!
nonExtendedSubInstances

	"very inefficient algorithm..."

	| all |

	all _ self allSubInstances. 
	^ all select: [:ms | all noneSatisfy: [:ums | ums extends: ms]]! !

!LambdaWrapper methodsFor: 'as yet unclassified' stamp: 'spfa 5/15/2006 21:53'!
object: anObject

	object _ anObject! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 2/8/2007 14:39'!
ofVariables: anArrayOfSlots

	| lms ss |
	
	lms _ self copy.
	lms slots: (ss _ anArrayOfSlots collect: [:s | s slot]).
	lms freeInnerSlots: ss.
	^ lms ! !

!Object methodsFor: '*FunctionalTalk' stamp: 'spfa 12/11/2006 14:29'!
ofVariables: anArrayOfSlots
	
	anArrayOfSlots ifEmpty: [^ self].

	^ self lambda ofVariables: anArrayOfSlots! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 5/11/2006 19:23'!
o: aMessageSend

	^ self composeWith: aMessageSend

	! !

!LambdaMessageSend methodsFor: 'parameters' stamp: 'spfa 2/2/2007 20:44'!
parameters

	^ self boundSlots select: [:bslot | bslot default ~~ Lambda]
		
! !

!LambdaMessageSend methodsFor: 'parameters' stamp: 'spfa 2/4/2007 12:41'!
parametrize: aSymbol default: bObject

	(self boundSlots detect: [:rs | rs id == aSymbol] ifNone: [^ self]) 
		default: bObject.
	self changed! !

!LambdaMessageSend methodsFor: 'copying' stamp: 'spfa 1/19/2007 16:50'!
postCopy

	receiver isLambda ifTrue: [receiver _ receiver lambdaClone].
	selector isLambda ifTrue: [selector _ selector lambdaClone].
	arguments isLambda ifTrue: [arguments _ arguments lambdaClone]
		ifFalse: [arguments do: 
					[:arg | arg isLambda ifTrue: [arg _ arg lambdaClone]]].
	slots _ slots copy

		! !

!LambdaMessageSend methodsFor: 'printing' stamp: 'spfa 1/19/2007 16:07'!
printOn: stream 

	| wrapflag bSlots |

	bSlots _ self boundSlots.

	bSlots ifNotEmpty:
	[
		bSlots do: [:s | stream nextPut: $L; nextPutAll: s id; nextPut: $.].
		stream nextPut: ${.
	].

	wrapflag _ (selector isLambda 
					or: [selector notNil and: [selector numArgs > 0]] 
					or: [arguments isLambda])
				and: [bSlots isEmpty].

	wrapflag ifTrue: [stream nextPut: $(].
	self maybeWrap: receiver onStream: stream using: #printString.
	selector ifNotNil: [
 	    stream space.
		(arguments isLambda not and: [arguments isEmpty]) ifTrue: [
				selector isLambda ifTrue:[
						stream nextPutAll: 'perform: '.
						selector printOn: stream]
					ifFalse: [stream nextPutAll: selector]]
			ifFalse: [
				(selector isLambda or: [arguments isLambda]) ifTrue:[
						stream nextPutAll: 'perform:'; space.
						selector printOn: stream.
						stream space; nextPutAll: 'withArguments:'; space.
						arguments printOn: stream]
					ifFalse: [
				  		arguments with: selector keywords do: [:arg :word |
							stream nextPutAll: word; space.
							self maybeWrap: arg onStream: stream using: #printString.
							stream space].
						stream skip: -1]]].
	wrapflag ifTrue: [stream nextPut: $)].

	bSlots ifNotEmpty: [stream nextPut: $}].! !

!LambdaSlot methodsFor: 'printing' stamp: 'spfa 5/22/2006 10:44'!
printOn: stream 

	stream nextPut: $<; nextPutAll: id asString; nextPut: $>.

! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 2/9/2007 14:55'!
privateReceiver: anObject

	receiver _ anObject
	! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/15/2006 21:54'!
quoted

	^ LambdaWrapper around: self! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 2/9/2007 14:54'!
receiver: anObject

	anObject isLambda ifTrue: [receiver _ anObject lambdaClone]
		ifFalse: [receiver _ anObject].
	! !

!LambdaMessageSend class methodsFor: 'instance creation' stamp: 'spfa 2/8/2007 14:39'!
receiver: anObject selector: aSymbol arguments: anArray

	^ (super receiver: anObject selector: aSymbol arguments: anArray) 
			ensureUnambigousSlotIds ! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 1/19/2007 13:16'!
reduceWithAllNonLambda: args

	^ self compiled valueWithArguments: args
! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 1/17/2007 20:39'!
reduceWithAll: args

	^ self reduceWithSlots: (self argumentsFromList: args)


! !

!LambdaSlot methodsFor: 'reduction' stamp: 'spfa 5/14/2006 14:18'!
reduceWithAll: args

	| x |

	args size ~= 1 ifTrue: [self error: 'Bad number of arguments'].

	((x _ args first) == Lambda
	 or: [x isLambdaSlot and: [x id isNil]]) ifTrue: [^ self].

	^ x
! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 5/11/2006 10:17'!
reduceWithNonLambda: arg

	^ self reduceWithAllNonLambda: {arg}


! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 2/8/2007 18:52'!
reduceWithSlots: associations

	  | dict |
	
	self hasCompiledForm ifFalse: 
		[^ self substituteSlots: associations reduce: true].

	dict _ IdentityDictionary withAssociations: associations.
	dict keys = self boundVariables asIdentitySet
	ifTrue:
	[
		| args |
		args _ self boundSlots collect: [:bs | dict at: bs id].
		(args anySatisfy: [:arg | arg isLambda or: [arg == Lambda]])
			ifFalse: [^ (compiledForm valueWithArguments: args) lambdaValue]
	].

	^ self substituteSlots: associations reduce: true
! !

!LambdaSlot methodsFor: 'reduction' stamp: 'spfa 12/8/2006 15:13'!
reduceWithSlots: association

	^ self substituteSlots: association
! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 12/10/2006 16:43'!
reduceWithSlot: anAssociation

	^ self reduceWithSlots: {anAssociation}


! !

!LambdaSlot methodsFor: 'reduction' stamp: 'spfa 5/23/2006 14:21'!
reduceWithSlot: anAssociation

	^ self substituteSlot: anAssociation
! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 5/9/2006 14:18'!
reduceWith: arg

	^ self reduceWithAll: {arg}


! !

!LambdaMessageSend methodsFor: 'testing' stamp: 'spfa 1/27/2007 10:33'!
references: anObject

	^ (super references: anObject) or: 
	[
		(receiver isMessageSend and: [receiver references: anObject])
		or: [selector isMessageSend and: [selector references: anObject]]
		or: [arguments isMessageSend and: [arguments references: anObject]]
		or: [arguments notNil
				and: [arguments anySatisfy: [:arg | 
						arg isMessageSend and: [arg references: anObject]]]]
	]! !

!MessageSend methodsFor: '*FunctionalTalk' stamp: 'spfa 1/27/2007 10:33'!
references: anObject

	^ receiver == anObject
		or: [selector == anObject]
		or: [arguments notNil and: [arguments identityIncludes: anObject]]! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 1/19/2007 16:50'!
selector: anObject

	anObject isLambda ifTrue: [selector _ anObject lambdaClone]
		ifFalse: [selector _ anObject].
	! !

!LambdaSlot class methodsFor: 'API' stamp: 'spfa 5/9/2006 14:39'!
selector: aSymbol 

	^ self selector: aSymbol arguments: #()! !

!LambdaSlot class methodsFor: 'API' stamp: 'spfa 12/6/2006 13:44'!
selector: aSymbol arguments: anArray

	^ self selector: aSymbol arguments: anArray slotId: #receiver! !

!LambdaSlot class methodsFor: 'API' stamp: 'spfa 12/11/2006 14:30'!
selector: aSymbol arguments: anArray slotId: bSymbol

	| slot |

	^ (LambdaMessageSend receiver: (slot _ LambdaSlot id: bSymbol) selector: aSymbol arguments: anArray) ofVariables: {slot}! !

!LambdaSlot class methodsFor: 'API' stamp: 'spfa 5/9/2006 14:39'!
selector: aSymbol argument: anObject

	^ self selector: aSymbol arguments: {anObject}! !

!LambdaSlot class methodsFor: 'API' stamp: 'spfa 12/6/2006 13:44'!
selector: aSymbol argument: anObject slotId: bSymbol

	^ self selector: aSymbol arguments: {anObject} slotId: bSymbol! !

!LambdaSlot class methodsFor: 'API' stamp: 'spfa 12/6/2006 13:44'!
selector: aSymbol  slotId: bSymbol

	^ self selector: aSymbol arguments: #() slotId: bSymbol! !

!Point methodsFor: '*FunctionalTalk-override' stamp: 'spfa 1/17/2007 21:20'!
setX: xValue setY: yValue

	xValue isNumber ifFalse: [self error].
	yValue isNumber ifFalse: [self error].

	x _ xValue.
	y _ yValue! !

!Browser methodsFor: '*FunctionalTalk-override' stamp: 'spfa 1/27/2007 10:43'!
shiftedMessageListMenu: aMenu
	"Fill aMenu with the items appropriate when the shift key is held down"

	Smalltalk isMorphic ifTrue: [aMenu addStayUpItem].
	aMenu addList: #(
		('method pane' 							makeIsolatedCodePane)
		('tile scriptor'							openSyntaxView)
		('toggle diffing (D)'						toggleDiffing)
		('implementors of sent messages'			browseAllMessages)
		-
		('local senders of...'						browseLocalSendersOfMessages)
		('local implementors of...'				browseLocalImplementors)
		-
		('lambda references to...'			exploreLambdaReferences)
		-
		('spawn sub-protocol'					spawnProtocol)
		('spawn full protocol'					spawnFullProtocol)
		-
		('sample instance'						makeSampleInstance)
		('inspect instances'						inspectInstances)
		('inspect subinstances'					inspectSubInstances)).

	self addExtraShiftedItemsTo: aMenu.
	aMenu addList: #(
		-
		('change category...'					changeCategory)).

	self canShowMultipleMessageCategories ifTrue: [aMenu addList:
		 #(('show category (C)'						showHomeCategory))].
	aMenu addList: #(
		-
		('change sets with this method'			findMethodInChangeSets)
		('revert to previous version'				revertToPreviousVersion)
		('remove from current change set'		removeFromCurrentChanges)
		('revert & remove from changes'		revertAndForget)
		('add to current change set'				adoptMessageInCurrentChangeset)
		('copy up or copy down...'				copyUpOrCopyDown)
		-
		('fetch documentation'					fetchDocPane)
		('more...' 								unshiftedYellowButtonActivity)).
	^ aMenu
! !

!LambdaMessageSend methodsFor: 'atomic access' stamp: 'spfa 2/2/2007 20:44'!
slot

	self isAtomic ifFalse: [self error: 'not an atomic lambda'].

	^ receiver! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 12/6/2006 14:30'!
slot

	^ self! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 12/6/2006 13:31'!
slotIds

	^ self boundSlots collect: [:bslot | bslot id].! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 12/6/2006 13:31'!
slotIds: anArray

	self boundSlots with: anArray do: [:bslot :id | bslot id: id].! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 12/6/2006 13:37'!
slotId: aSymbol newId: bSymbol

	(self boundSlots detect: [:bslot | bslot id == aSymbol] ifNone: [^ self])
		id: bSymbol! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 2/3/2007 20:27'!
slots

	^ Array streamContents: [:str |
		receiver isLambda ifTrue: [str nextPutAll: receiver slots].
		selector isLambda ifTrue: [str nextPutAll: selector slots].	
		arguments isLambda ifTrue: [str nextPutAll: arguments slots]
			ifFalse: [arguments do: [:a | 
							a isLambda ifTrue: [str nextPutAll: a slots]]]]
! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 5/11/2006 15:35'!
slots

	^ {self}! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 1/17/2007 19:15'!
slots: anArray

	slots _ anArray.
	! !

!LambdaMessageSend methodsFor: 'parameters' stamp: 'spfa 2/4/2007 11:09'!
staticDefaultForm

	"parameters defaults can themselves be lambda expressions...
see #defaultForm"

	^ (self boundSlots select: [:slot | slot default ~~ Lambda] 
			thenCollect: [:slot | slot id -> slot default value]) @@>> self

! !

!LambdaMessageSend methodsFor: 'parameters' stamp: 'spfa 2/4/2007 22:03'!
staticDefaultFor: aSymbol

	^ (self defaultFor: aSymbol) lambdaValue
! !

!LambdaMessageSend methodsFor: 'compilation' stamp: 'spfa 2/8/2007 16:10'!
storeDataOn: aDataStream

	self eraseCompiledForm.
	super storeDataOn: aDataStream.


! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 12/10/2006 19:29'!
substituteSlots: associations

	^ self substituteSlots: associations reduce: false! !

!LambdaSlot methodsFor: 'reduction' stamp: 'spfa 12/8/2006 15:19'!
substituteSlots: associations

	| v dict |

	dict _ IdentityDictionary withAssociations: associations.

	v _ dict at: id ifAbsent: [^self].

	v == Lambda ifTrue: [^ self].

	^ v
! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 12/11/2006 13:38'!
substituteSlots: associations reduce: flag

	^ self substituteSlots: associations reduce: flag inline: false! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 2/10/2007 11:43'!
substituteSlots: associations reduce: flag inline: inflag

	| argsSlots rrec rsel rargs dict val nbs |

"==============================================="
"move this elsewhere !!"
	(associations isKindOf: IdentityDictionary)
	ifFalse: 
[
	| allArgsSlots slotsDictionary allSlots |

	allArgsSlots _ ((associations select: [:a | 
							a value isLambda and: [a value isLambdaSlot not]])
						inject: #() into: [:s :a | s, a value boundSlots, a value slots])
					asIdentitySet.

	allSlots _ (self boundSlots, self slots) asIdentitySet.

	slotsDictionary _ IdentityDictionary new.
	allArgsSlots do: [:s |
		slotsDictionary add: s -> ((allSlots identityIncludes: s)
				ifTrue: [s copy] ifFalse: [s])
	].

	dict _ IdentityDictionary withAssociations: (associations collect: [:a |
		(a value isLambda
			and: [a value isLambdaSlot not])
		ifTrue: [a key -> (a value withNewSlots: slotsDictionary)]
		ifFalse: [a]]).
]
ifTrue: [dict _ associations].
"==============================================="

	argsSlots _ IdentityDictionary new.
	(dict associations select: [:assoc | assoc value isLambda])
		do: [:assoc | argsSlots add: (assoc key -> assoc value boundSlots)].

	rrec _ receiver isLambda 
				ifTrue: [receiver substituteSlots: dict reduce: flag inline: inflag] 
				ifFalse: [receiver].

	rsel _ selector isLambda 
				ifTrue: [selector substituteSlots: dict reduce: flag inline: inflag] 
				ifFalse: [selector].

	rargs _ arguments isLambda 
				ifTrue: [arguments substituteSlots: dict reduce: flag inline: inflag] 
				ifFalse: [arguments collect: [:arg | 
								arg isLambda 
							ifTrue: [arg substituteSlots: dict reduce: flag inline: inflag] 
							ifFalse: [arg]]].

	nbs _ Array streamContents: [:str |
				self boundSlots do: [:bs |
					((dict includesKey: bs id) not
						or: [(dict at: bs id) == Lambda])
						ifTrue: [str nextPut: bs]
						ifFalse: [(inflag and: [argsSlots includesKey: bs id])
									ifTrue: [str nextPutAll: (argsSlots at: bs id)]]]].

	nbs _ Array streamContents: [:str |
		nbs do: [:s | (str contents identityIncludes: s) ifFalse: [str nextPut: s]]]. 

	val _ self class receiver: rrec selector: rsel	arguments: rargs.

	inflag ifTrue:
		[val freeInnerSlots: (argsSlots values inject: #() into: [:this :all | all, this])].

	val _ val ofVariables: nbs.

	flag ifTrue: [val _ val valueRecursive: false].

	^ val

! !

!LambdaSlot methodsFor: 'reduction' stamp: 'spfa 1/18/2007 20:47'!
substituteSlots: associations reduce: flag inline: inflag

	^ self substituteSlots: associations
! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 12/10/2006 18:02'!
substituteSlot: anAssociation

	^ self substituteSlots: {anAssociation}


! !

!LambdaSlot methodsFor: 'reduction' stamp: 'spfa 5/23/2006 14:21'!
substituteSlot: anAssociation

	(id = anAssociation key) ifTrue: [^ anAssociation value]
! !

!LambdaMessageSend methodsFor: 'reducting' stamp: 'spfa 12/11/2006 13:47'!
substitute: args

	^ self substituteSlots: (self argumentsFromList: args)


! !

!LambdaSlot methodsFor: 'reduction' stamp: 'spfa 5/23/2006 11:11'!
substitute: args

	args size ~= 1 ifTrue: [self error: 'Bad number of arguments'].	
	args first == Lambda ifTrue: [^ self].
	^ args first
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 5/13/2006 22:17'!
testArithmetic

	self assert: (2 + Lambda x) isLambda.
	self assert: (1/(Lambda x + 1)) isLambda.
	self assert: (1/Lambda x) isLambda! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 5/11/2006 18:38'!
testArity1

	| f g f2 ff sumf |

	f _ 56 lambda: #+ .
	g _ f sqrt.
	f2 _ f * 2.
	ff _ f * f.
	sumf _ ff + f2 + g.

	self assert: (sumf <~ 8) = 4232.
	self assert: (f <~ 4) = 60.
	self assert: (g <~ 8) = 8.	
	self assert: (f2 <~ 4) = 120.
	self assert: (ff <~ 4) = 3600.

! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/3/2007 20:54'!
testArity2

	| f t |

	f _ 56 lambda: #+.
	t _ (f sqrt) + Lambda x.

	self assert: t boundSlots size = 2.
	self assert: (t <~~ {8 . 2}) = 10.
	self assert: (t + f) boundSlots size = 2.
	self assert: ((t + f) <~~ {8 . 2}) = 74.
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 15:34'!
testArity3

	| f g t h |

	f _ 56 lambda: #+.
	g _ 10 lambda: #-.
	t _ f + g.

	self assert: t boundSlots size = 2.
	self assert: (t <~~ {8 . 8}) = 66.

	h _ Point lambda: #x:y:.
	t _  h r.

	self assert: (t <~ 0 <~ 1) = 1.
	self assert: ((f + t) <~ 4 <~ 10 <~ 0) = 70
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 5/11/2006 18:37'!
testArityPreservation

	| f t g |

	f _ LambdaSlot selector: #+ arguments: {56}.
	t _ (f sqrt) + 2.
	g _ ((f sqrt) + 2) * f.

	self assert: ((t <~ 8) closeTo: 10).
	self assert: ((g <~ 8) closeTo: 640).	
	self assert: ((t <~ -56) closeTo: 2).
	self assert: ((g <~ -55) closeTo: 3).	
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/8/2007 13:19'!
testAtomicLMS

	| f g a b c d |

	a _ Lambda a.
	b _ Lambda b.
	c _ Lambda c.
	d _ Lambda d.

	f _ b ofVariables: {a . b . c . d}.
	self assert: f isAtomic.
	self assert: (f <<= (#b -> Lambda u)) boundVariables = #(a u c d).

	g _ a + b + c + d.
	self deny: g isAtomic.
	self assert: (g <<= (#b -> Lambda u)) boundVariables = #(a u c d).
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 12/11/2006 15:08'!
testBasics

	"defining a message send from an arbitrary object:"
	self assert: 5 lambda isMessageSend.
	self assert: 5 lambda value = 5.

	"defining a message send from an object and message:"
	self assert: (4 lambda + 1) isMessageSend.
	self assert: (4 lambda + 1) value = 5.

	"defining a lambda expression from an object and selector only"
	self assert: (4 lambda: #+) isLambda.
	self assert: (4 lambda: #+) isMessageSend.
	"this can not be evaluated as a plain message send, as it would evaluate to itself"
	self assert: (4 lambda: #+) value = (4 lambda: #+). 


 	"an argument can be provided via #substitute:"
	self assert: ((4 lambda: #+) substitute: {5}) isMessageSend.
	"this can now be evaluated"
	self assert: ((4 lambda: #+) substitute: {5}) value = 9 .
	self assert: ((4 lambda: #+) substitute: {5}) printString = '(4 + 5)' .

	"shorter API:"
	self assert: ((4 lambda: #+) <@ 5) printString = '(4 + 5)'.
	self assert: ((1+ (4 lambda: #+)) <@ 5) printString = '(1 + (4 + 5))'.

	"giving an argument and getting the corresponding value is called reduction. this can be done in one step"
	self assert: ((4 lambda: #+) reduceWith: 5) = 9 .
	"shorter API :"
	self assert: ((4 lambda: #+) <~ 5) = 9 .

! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 12/11/2006 14:04'!
testBasics2

	| f g r |

	f _ 4 lambda: #+.

	"a lambda expression can be expanded by sending it any messages"
	g _ (f + 10) * 2.
	self assert: g isLambda.
	self assert: (g <~ 1) = 30.

	"it can be composed with another lambda expression"
	self assert: (f ~> g) isLambda.
	self assert: ((f => g) <~ 1) = 38.

	"actually ~> is just <~ the other way round"
	self assert: (g <= f <~ 1) = 38.
	self assert: (1 ~> f => g) = 38.
	self assert: (f => g <~ 1) = 38.   

	"to see what (f => g) really is, print it"
	self assert: (f => g) printString = 'L1.{((4 + (4 + <1>)) + 10) * 2}'.

	self assert: f printString = 'L1.{4 + <1>}'.
	self assert: g printString = 'L1.{((4 + <1>) + 10) * 2}'.
	self assert: (g => f) printString = 'L1.{4 + (((4 + <1>) + 10) * 2)}'.

	"<1>, <2>, .. are the default labels (id) for lambda slots
	it is possible to define arbitrary ids"
	f _ 4 lambda: #+ slotId: #'surprenant, non ?'.
	self assert: f printString = 'Lsurprenant, non ?.{4 + <surprenant, non ?>}' .

	"this is especially useful for expressions with several arguments"
	r  _ Rectangle lambda: #left:right:top:bottom: 
					slotIds: #(left right top bottom).
	self assert: r printString =
		'Lleft.Lright.Ltop.Lbottom.{Rectangle left: <left> right: <right> top: <top> bottom: <bottom>}' .

	"we can reduce the number of arguments by providing values for specific slots, using slot ids"
	r _ r reduceWithSlots: {#left -> 50 . #right -> 60}.
	self assert: 
	r <~~ {0 . 100} = (Rectangle left: 50 right: 60 top: 0 bottom: 100).
	self assert: 
	r printString = 'Ltop.Lbottom.{Rectangle left: 50 right: 60 top: <top> bottom: <bottom>}'.

	"we can substitute a lambda expression in a slot"
	r _ r inlineSlot: (#bottom -> (100 lambda: #- slotId: #'bottom offset')).
	self assert: 
	r slotIds = #(top #'bottom offset').
	self assert: 
	r <~~ {0 . 0} = (Rectangle left: 50 right: 60 top: 0 bottom: 100).
	self assert: r printString = 'Ltop.Lbottom offset.{Rectangle left: 50 right: 60 top: <top> bottom: (100 - <bottom offset>)}'.

	"<<~ and <<~~ are shortcuts for reduceWithSlot: and reduceWithSlots:
you can also use the backward versions ~>> and ~~>>"
	r _ Rectangle lambda: #left:right:top:bottom: 
			slotIds: #(left right top bottom).
	r _ r <<~ (#right -> 60).
	r _ {#left -> 50 . #bottom -> (100 lambda: #-)} ==>> r.
	self assert: 
	{0 . 0} ~~> r = (Rectangle left: 50 right: 60 top: 0 bottom: 100).
	! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 1/19/2007 14:44'!
testBasics3

	| f g h k j m |

	"lambda abstraction can be based on the receiver of an expression"
	f _ LambdaSlot + 5.
	self assert: (f <~ 1) = 6.
	self assert: f printString = 'Lreceiver.{<receiver> + 5}'.

	"we can also abstract both receiver and arguments at the same time"
	g _ (LambdaSlot lambda: #+) .
	self assert: (g <~~ {2 . 3}) = 5.
	self assert: g printString = 'Lreceiver.L1.{<receiver> + <1>}'.

	"this is equivalent to"
	g _ LambdaMessageSend 
			receiver: (LambdaSlot id: #receiver)
			selector: #+ 
			arguments: {LambdaSlot id: #argument}.
	self assert: (g <~~ {2 . 3}) = 5.

	"the selector can be abstracted too..."
	h _ 5 lambda: {5}.
	self assert: (h <~ #+) = 10.
	self assert: (h <~ #*) = 25.
	self assert: h printString = 'Lselector.{5 perform: <selector> withArguments: #(5)}'.

	"abstracting both receiver and selector"
	j _ (LambdaSlot lambda: {5}) .
	self assert: (j <~~ {2 . #+}) = 7.
	self assert: (j <~~ {1 . #-}) = -4.
	self assert: j printString = 'Lreceiver.Lselector.{<receiver> perform: <selector> withArguments: #(5)}'.

	"abstracting both selector and arguments"
	m _ 5 lambdaMessage.
	self assert: (m <~~ {#+ . {3}}) = 8.
	self assert: (m <~~ {#raisedTo: . {2}}) = 25.
	self assert: (m <~~ {#adaptToFraction:andSend: . {1/2 .  #+}}) = (11/2).
	self assert: m printString = 'Lselector.Larguments.{5 perform: <selector> withArguments: <arguments>}'.

	"here is the full monty"
	k _ LambdaSlot lambdaMessage.
	self assert: (k <~~ {2 . #+ . {3}}) = 5.
	self assert: (k <~~ {'hello' . #padded:to:with: . {#right . 10 . $!!}}) = 'hello!!!!!!!!!!'.
	self assert: k printString = 'Lreceiver.Lselector.Larguments.{<receiver> perform: <selector> withArguments: <arguments>}' .

	"this is equivalent to"
	k _ LambdaMessageSend 
			receiver: (LambdaSlot id: #receiver) 
			selector: (LambdaSlot id: #selector) 
			arguments: (LambdaSlot id: #arguments).
	self assert: (k <~~ {2 . #+ . {3}}) = 5.
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 1/19/2007 19:25'!
testBasics4

	| x f g h z p |

	"slots are shared by default when combining functions"
	x _ Lambda x.
	f _ x * (1+x).
	self assert: f boundSlots size = 1. "this means the two x above will take the same one argument"
	self assert: f allSlotIds = #(x x).
	self assert: f slotIds = #(x).
	self assert: 10 ~> f = 110.
	
	"slots are not shared when copying"
	g _ x * (1+ x lambdaCopy).
	self assert: g boundSlots size = 2. "there is now TWO arguments"
	self assert: g allSlotIds = #(x1 x).
	self assert: g slotIds = #(x1 x).
	self assert: {10 . 2} ~~> g = 30.

	"slots built independently stay independent"
	h _ Lambda y * (1+ Lambda y).
	self assert: h boundSlots size = 2.  "the two Lambda y define different slots"
	self assert: h allSlotIds = #(y1 y).
	self assert: h slotIds = #(y1 y).
	self assert: {10 . 2} ~~> h = 30.

	"note that it is always possible to redefine the slots and change the arity"
	z _ Lambda z.
	p _ {z . z} ==> h.
	self assert: p boundSlots size = 1.
	self assert: p allSlotIds = #(z z).
	self assert: p slotIds = #(z).
	self assert: 10 ~> p = 110.

! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 5/15/2006 22:55'!
testBasics5

	"a lambda expression is an instance of LambdaMessageSend.
all messages send to it are swallowed in the expression, except if LambdaMessageSend can understand them"
	self assert: ((4 lambda + 5) sin isInfinite not) isMessageSend.
	self deny: ((4 lambda + 5) sin = 0) isMessageSend.
	"this because #= is understood by LambdaMessageSend"

	"#quoted is more powerful than #lambda because it catches almost all selectors. the only exceptions are the selectors implemented by ProtoObject.
	its semantic is different though: it returns a LambdaWrapper, ready to catch the following selector. so it must be followed by a selector in order to give a lambda expression"
	self assert: 4 lambda isMessageSend.
	self deny: 4 quoted isMessageSend = true.
	"note that we can not just do 
	self deny: 4 quoted isMessageSend
	because (4 quoted isMessageSend) IS a MessageSend !!" 
	self deny: (4 lambda = 4) isMessageSend.
	self assert: (4 quoted = 4) isMessageSend.
	"#== is implemented by ProtoObject so we cannot catch it at all"
	self deny: (4 lambda == 4) isMessageSend.
	self deny: (4 quoted == 4) isMessageSend. 
	"here is the way to do it"
	self assert: (4 lambda: #== with: 4) isMessageSend.! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 2/9/2007 15:00'!
testBenchmarks

	| bk x y f tblock t |

	x _ Lambda x.
	y _ Lambda y.
	f _ ((x*x) + (y*y)) sqrt.
	f compile.

	bk _ [:a :b | ((a*a) + (b*b)) sqrt].

	"tblock is the duration taken by bk to evaluate 100000 times:"
	Smalltalk garbageCollect.
	tblock _ [100000 timesRepeat: [bk value: 3 value: 4]] durationToRun. 

	"the default way to reduce a lambda expression is quite slow:
	here it is about 600 times slower than its equivalent block
	(although this does not make much sense as both do not perform the same operation at all)"
	Smalltalk garbageCollect.
	t _ [160 timesRepeat: [f <~~ {3 . Lambda}]] durationToRun.
	self assert: ((t * 1.4) > tblock) & ((t * 0.7) < tblock).

	"now if we are not actually substituting but simply evaluating the lambda expression, it gets better. 
	here we perform exactly the same operation as the block: it is roughly 40 times slower"
	Smalltalk garbageCollect.
	t _ [2500 timesRepeat: [f <~~ {3 . 4}]] durationToRun.
	self assert: ((t * 1.4) > tblock) & ((t * 0.7) < tblock).

	"if the function is to be evaluated with plain, non-lambda arguments, as in our example, then it is much better to use //> instead of ~~> because some expensive checks gets disabled.
	we are now only 2 times slower than the block"
	Smalltalk garbageCollect.
	t _ [50000 timesRepeat: [f <// {3 . 4}]] durationToRun.  
	self assert: ((t * 1.4) > tblock) & ((t * 0.7) < tblock).

	"even faster is the direct invocation of the compiled form of the function (which is a block itself): we eventually catch up with the block"
	Smalltalk garbageCollect.
	t _ [100000 timesRepeat: [f compiled value: 3 value: 4]] durationToRun. 
	self assert: ((t * 1.4) > tblock) & ((t * 0.7) < tblock).
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 5/15/2006 22:57'!
testBeware

	"lambda expressions expand by receiving message.
they do not expand when sent to other objects"
	self assert: ('abc' lambda, 'def') value = 'abcdef'.	
	self should: [('abc', 'def' lambda) value] raise: Error.
	self assert: ('abc' lambda, 'def' lambda) value = 'abcdef'.

	"as an exception, arithmetic operations do allow more freedom"
	self assert: (77 + 5 lambda) isMessageSend.
	self assert: (77 + 5 lambda) value = 82.
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 1/19/2007 21:37'!
testBeware2

	| f g h |

	"slots ids are changed silently if a name clash happen"

	f _ 5 lambda: #+ slotId: #x.
	g _ 1 lambda: #- slotId: #x.
	h _ f + g.
	self assert: h boundSlots size = 2.
	self assert: (h <~~ {5 . 1}) = 10.
	self assert:  h slotIds = #(x1 x).           

! !

!FunctionalTalkTest methodsFor: 'lambda calculus' stamp: 'spfa 12/11/2006 16:06'!
testBooleans

	| x y a b c atrue afalse ifthenelse |

	x _ Lambda x.
	y _ Lambda y.

	atrue _ x ofVariables: {x . y}.
	afalse _ y ofVariables: {x . y}.

	self assert: (#apple ~> afalse) printString = 'Ly.{<y>}'.
	self assert: (#apple ~> atrue) printString = 'Ly.{#apple}'.

	a _ Lambda a.
	b _ Lambda b.
	c _ Lambda c.

	ifthenelse _ (a quoted <~~ (Array quoted with: b with: c)) ofVariables: {a . b . c} .

	self assert: (ifthenelse <~~ {afalse . #apple . #banana}) == #banana.
	self assert: (ifthenelse <~~ {atrue . #apple . #banana}) == #apple.

	ifthenelse _ ((a quoted <~ b) quoted <~ c) ofVariables: {a . b . c} .

	self assert: (ifthenelse <~~ {afalse . #apple . #banana}) == #banana.
	self assert: (ifthenelse <~~ {atrue . #apple . #banana}) == #apple.

	ifthenelse _ (c quoted ~> (b quoted ~> a)) ofVariables: {a . b . c} .

	self assert: (ifthenelse <~~ {afalse . #apple . #banana}) == #banana.
	self assert: (ifthenelse <~~ {atrue . #apple . #banana}) == #apple.
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 1/19/2007 19:21'!
testBoundSlots

	| f x y |
	
	x _ Lambda x.

	self assert: x printString = 'Lx.{<x>}'.
	self assert: x slotIds = #(x).
	self assert: x freeSlots = #().

	f _ Lambda x + 1.
	self assert: f printString = 'Lx.{<x> + 1}'.
	self assert: f slotIds = #(x).
	self assert: (f <~ 1) = 2.
	self assert: f freeSlots = #().

	y _ Lambda y.

	f _ (x + 1) ofVariables: {x . y}.
	self assert: f printString = 'Lx.Ly.{<x> + 1}'.
	self assert: f slotIds = #(x y).
	self assert: (f <~ 1 <~ #anything) = 2.
	self assert: (f <~ 1) printString = 'Ly.{1 + 1}'.
	self assert: f freeSlots = #().

	f _ (x + 1) ofVariables: {y . x}.
	self assert: f printString = 'Ly.Lx.{<x> + 1}'.
	self assert: f slotIds = #(y x).
	self assert: (f <~ #anything <~ 1) = 2.
	self assert: f freeSlots = #().

	f _ (x + 1) ofVariables: #().
	self assert: f printString = '(<x> + 1)'.
	self assert: f boundSlots = #().
	self assert: f freeSlotIds = #(x).
	self should: [f <~ 1] raise: Error.
	self assert: (f ofVariables: {x}) printString = 'Lx.{<x> + 1}'.
	self assert: ((f ofVariables: {x}) <~ x) printString = '(Lx.{<x>} + 1)'.! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 1/19/2007 19:46'!
testBoundSlots2

	| x y f |

	x _ Lambda x.
	y _ Lambda y.
	f _ y ofVariables: {x . y}. 

	self assert: (Lambda w <~ f) printString = 'Lx.Ly.{<y>}'.
	! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/8/2007 14:02'!
testBoundSlots3

	| u p q |

	u _ Lambda u.
	p _ Point lambda x: u y: u.
	q _ u @ u. 

	self assert: p printString = 'Lu.{Point x: <u> y: <u>}'.
	self assert: q printString = 'Lu.{<u> @ <u>}'.
	! !

!FunctionalTalkTest methodsFor: 'lambda calculus' stamp: 'spfa 2/10/2007 10:37'!
testChurchNumerals

"
	0 = Lf.Lx.x
	1 = Lf.Lx.(f)x
	2 = Lf.Lx.(f)(f)x
	3 = Lf.Lx.(f)(f)(f)x

	succ = Ln.Lf.Lx.(f)((n)f)x
	+ = Lm.Ln.Lf.Lx.((m)f)((n)f)x
	* = Lm.Ln.Lf.(m)(n)f
	zero? = Ln.((n)(true)false)true
	pred = Ln.(((n)Lp.Lz.((z)(succ)(p)true)(p)true)Lz.((z)0)0)false
"

	| x n m f n0 n1 n2 n3 succ sum "mul zerop pred
		atrue afalse ifthenelse a b c z y" |

	x _ Lambda x.
	n _ Lambda n.
	m _ Lambda m.
	f _ Lambda f.

	n0 _ x ofVariables: {f . x}.
	n1 _ f quoted <~ x ofVariables: {f . x}.
	n2 _ (f quoted <~ f) quoted <~x ofVariables: {f . x}.
	n3 _ ((f quoted <~ f) quoted <~ f) quoted <~x ofVariables: {f . x}.

	self assert: (n0 <~~ {Lambda x + 1 . 0}) = 0.
	self assert: (n1 <~~ {Lambda x + 1 . 0}) = 1.
	self assert: (n2 <~~ {Lambda x + 1 . 0}) = 2.
	self assert: (n3 <~~ {Lambda x + 1 . 0}) = 3.

	succ _  ((n quoted <~ f) quoted <~ (f quoted <~ x)).

	self assert: (succ <~ n0) printString = n1 printString.
	self assert: (succ <~ n1) printString = n2 printString.
	self assert: (succ <~ n2) printString = n3 printString.
	
"... I'm still struggling with the others operators ... to be continued :)"

	sum _ ((m quoted <<~ (#x quoted -> n)) quoted ofVariables: {f . x}) ofVariables: {m . n}.

	sum <~~ {n1 . n3} .
	sum <~~ {n2 . n3} .
	sum <~~ {n2 . n2} .

"	self assert: false." "to do.."
"
	mul  _ ((m quoted <~ n) quoted ofVariables: {f . x}) ofVariables: {m . n}. 

 	mul <~~ {n1 . n3} .
	mul <~~ {n2 . n3} .
	mul <~~ {n2 . n2} .
"
"	self assert: false." "to do.."
"
	z _ Lambda z.
	y _ Lambda y.
	atrue _ z ofVariables: {z . y}.
	afalse _ y ofVariables: {z . y}.

	a _ Lambda a.
	b _ Lambda b.
	c _ Lambda c.

	ifthenelse _ (a quoted <~~ (Array quoted with: b with: c)) ofVariables: {a . b . c} .

	zerop _ (n quoted <~~ (Array quoted with: atrue with: afalse)) ofVariables: {n}.

	zerop <~ n0.
	zerop <~ n1.
	zerop <~ n2.
 
self assert: false."! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 2/9/2007 20:02'!
testClassLambda

	"Lambda acts as a shortcut for the creation of an atomic LambdaMessageSend (e.g. the identity function)"

	| f |

	self assert: Lambda x <~ 4 = 4.
	self assert: {4} ~~> Lambda z = 4.
	self deny: Lambda x <@ 4 = 4.
	self assert: (Lambda x <@ 4) value = 4.

	self assert: (Lambda x <@ Lambda y) printString = 'Ly.{<y>}'.
	self assert: ({Lambda} ~~> Lambda z) printString = 'Lz.{<z>}'.
	self assert: ({Lambda x} ~~> Lambda z) printString = 'Lx.{<x>}'.

	self assert: (#x -> 45) ~>> Lambda x = 45.
	self assert: Lambda x <<~ (#x -> 45) = 45.

	f _ Lambda x * Lambda y.

	self assert: (f o: Lambda x) printString = 'Lx.Ly.{(<x> * <y>)}'.
	self assert: (Lambda x o: f) printString = 'Lx.Ly.{<x> * <y>}'.! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/6/2006 13:54'!
testCompilation

	| cat |

	cat _  '--test leftover (remove it all)--'.

	((Rectangle lambda: #left:right:top:bottom:)
		<<~~ {#'3' -> 500 . #'4' -> 600}) 
		slotIds: #(gauche droite);
		compileAs: #gauche:droite: in: Rectangle classified: cat.

	self assert: (Rectangle gauche: 45 droite: 177) =  (45@500 corner: 177@600).

	Rectangle class removeCategory: cat! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/6/2006 13:54'!
testCompilation2

	| cat |

	cat _  '--test leftover (remove it all)--'.

	((Rectangle lambda: #left:right:top:bottom:)
		<<~~ {#'3' -> 500 . #'4' -> 600}) 
		slotIds: {'gauche' . 'droite' };
		compileIn: Rectangle classified: cat.

	self assert: (Rectangle gauche: 45 droite: 177) =  (45@500 corner: 177@600).

	Rectangle class removeCategory: cat! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 1/19/2007 22:28'!
testCompilation3

	| g cat |

	cat _  '--compiled example (remove it if you want)--'.

	g _ LambdaMessageSend 
			receiver: (LambdaSlot id: #receiver)
			selector: (LambdaSlot id: #selector) 
			arguments: {LambdaSlot id: #argument}.

	g compileIn: LambdaSlot classified: cat.

	self assert: (LambdaSlot receiver: 10 selector: #/ argument: 2) = 5.

	LambdaSlot class removeCategory: cat! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 5/10/2006 15:06'!
testCompilation4

	| cat |

	cat _  '--compiled example (remove it if you want)--'.

	LambdaSlot lambdaMessage compileIn: LambdaSlot classified: cat.

	self assert: (LambdaSlot receiver: 10 selector: #/ arguments: {2}) = 5.

	LambdaSlot class removeCategory: cat! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/8/2007 14:18'!
testCompiledForm

	| f g morph |

	f _ ((Lambda x raisedTo: 2) + (Lambda y raisedTo: 2)) sqrt.
	self deny: f hasCompiledForm.
	f compile.
	self assert: f hasCompiledForm.
	self assert: f lambdaCopy hasCompiledForm.

	g _ f lambdaCopy <<~ (#y -> 0).
	g compile.
	self assert: g hasCompiledForm.

	self assert: (g <~ 2) = 2.
	self assert: (f <// {3 . 4}) = 5.

	morph _ Morph new.
	f _ morph lambda: #position:.
	f compile.
	self assert: f hasCompiledForm.
	self assert: (f <~ (0@0)) == (f <~ (0@0)) . 
	f <~ (55@12).       
	self assert: morph position = (55@12).


! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 14:05'!
testComposition

	| f g gf ggf gggf fgggf |

	f _ 56 lambda: #+ .
	g _ 10 lambda: #* .
	gf _ f => g.
	ggf _ gf => g.
	gggf _ f => g => g => g.
	fgggf _ f <= g <= ggf.

	self assert: ((4 ~> gf ) = 600).
	self assert: ((4 ~> ggf ) = 6000).
	self assert: ((4 ~> gggf ) = 60000).
	self assert: ((4 ~> fgggf ) = 60056)! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 14:06'!
testComposition2

	| p mp |

	p _ Point lambda: #x:y: .
	p _ p <== {1 lambda: #+ slotIds: {1} . 2 lambda: #* slotIds: {2}}. 

	self assert: p printString = 'L1.L2.{Point x: (1 + <1>) y: (2 * <2>)}'.
	self assert: (p <~~ {5 . 15}) = (6 @ 30).

	mp _ p <== {5 .  (Point lambda: #x:y:) r}.
	self assert: mp printString = 'L1.L2.{Point x: 6 y: (2 * (Point x: <1> y: <2>) r)}' . 
	self assert: (mp <~~ {0 . 1}) = (6 @ 2)
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 14:07'!
testComposition3

	| f t y |

	f _ LambdaSlot selector: #+ arguments: {56}.
	t _ f <== {3 lambda: #-}.
	y _ t <== {LambdaSlot selector: #+ arguments: {10}}.

	self assert: t printString = 'L1.{(3 - <1>) + 56}'.
	self assert: (t <~ 2) = 57.
	self assert: y printString = 'Lreceiver.{(3 - (<receiver> + 10)) + 56}'.
	self assert: (y <~ 2) = 47.
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 1/19/2007 22:06'!
testConstructor

	| f |

	f _ Lambda x + 12.

	self assert: (Compiler evaluate: f constructor) printString = 'Lx.{<x> + 12}'.

	f _ f*f.
	self assert: (Compiler evaluate: f constructor) printString 
					= 'Lx.{(<x> + 12) * (<x> + 12)}'.

	f _ f + Lambda y.
	self assert: (Compiler evaluate: f constructor) printString 
					= 'Lx.Ly.{((<x> + 12) * (<x> + 12)) + <y>}'.

	f _ f + f.
	self assert: (Compiler evaluate: f constructor) printString 
		= 'Lx.Ly.{(((<x> + 12) * (<x> + 12)) + <y>) + (((<x> + 12) * (<x> + 12)) + <y>)}'.

	self assert: {2 . 10} ~~> f = 412! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 15:43'!
testConstructor2

	| x f g |

	x _ Lambda x.
	f _ x + 12.
	g _ 2*x.
	self assert: (Compiler evaluate: f constructor) printString = 'Lx.{<x> + 12}'.
	self assert: (Compiler evaluate: g constructor) printString = 'Lx.{2 * <x>}'.
	self assert: (Compiler evaluate: (f+g) constructor) printString 
					= 'Lx.{(<x> + 12) + (2 * <x>)}'.
	self deny:  "beware of this !!"
		((Compiler evaluate: f constructor) + (Compiler evaluate: g constructor)) printString = 'Lx.{(<x> + 12) + (2 * <x>)}' .
	self assert:  "beware of this !!"
		((Compiler evaluate: f constructor) + (Compiler evaluate: g constructor)) printString = 'Lx1.Lx.{(<x1> + 12) + (2 * <x>)}' 

! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 1/19/2007 16:41'!
testCopy

	| f g |

	f _ ((Lambda x raisedTo: 2) + (Lambda y raisedTo: 2)) sqrt.
	g _ f lambdaCopy.

	self deny: (f slots identityIncludes: g slots first).
	self deny: (f slots identityIncludes: g slots second).

! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/2/2007 20:44'!
testDefaultForm

	| p q |

	p _ Lambda y + (10 * (Lambda x default: 42)). 
	self assert: p default printString = 'Ly.{<y> + 420}'.
	self assert: p defaultForm printString = 'Ly.{<y> + (10 * 42)}'.

	q _ (10 + (Lambda point default: 1@1) x).
	self assert: q default = 11.
	self assert: q defaultForm printString = '(10 + ((1@1) x))'.


! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 1/19/2007 21:37'!
testExample

	| x y f  g j |

	x _ Lambda x.
	y _ Lambda y.

	f _ ((x*x) + (y*y)) sqrt.

	self assert: f printString = 'Lx.Ly.{((<x> * <x>) + (<y> * <y>)) sqrt}'.
	self assert: {3 . 4} ~~> f = 5.

	g _ (#y -> 0) ~>> f.

	self assert: g printString = 'Lx.{((<x> * <x>) + 0) sqrt}'.

	j _ (#y -> (Lambda z1 + Lambda z2)) =>> f.

	self assert: j printString 
		= 'Lx.Lz1.Lz2.{((<x> * <x>) + ((<z1> + <z2>) * (<z1> + <z2>))) sqrt}'.
	self assert: {3 . 2 . 2} ~~> j = 5.
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 1/19/2007 19:16'!
testFreeSlots

	| f x |
	
	x _ Lambda x.

	self assert: x freeSlots = #().

	f _ Lambda x + 1.
	self assert: f freeSlots = #().

	self assert: (f ofVariables: #()) freeSlotIds = #(x).
	self assert: (f ofVariables: #()) unboundSlotIds = #(x).

	self assert: (f <~ Lambda x) freeSlots = #().
	self assert: (f <~ Lambda x) unboundSlotIds = #(x).! !

!FunctionalTalkTest methodsFor: 'lambda calculus' stamp: 'spfa 2/3/2007 21:00'!
testHighOrder

	| x y f |

	x _ Lambda x.
	y _ Lambda y.
	f _ Lambda f.
	
	self assert: ((((f quoted <~ (f quoted <~ x)) ofVariables: {f . x}) <~ (y*y))  <~ 5) = 625 .
! !

!FunctionalTalkTest methodsFor: 'lambda calculus' stamp: 'spfa 1/18/2007 20:02'!
testHighOrder2

	| y g w sum 
	specializedSum expandedSpecializedSum wronglyExpandedSpecializedSum |

	y _ Lambda y. "slot for a Number"
	g _ Lambda g. "slot for a function of one Number argument"
	w _ Lambda w. "slot for a Number"

	"we deal here with the high-order function
		 y + g(w)
	of arguments g, w and y"

	sum _ ( y + (g quoted <~ w) ) ofVariables: {g . w . y}.
	self assert: sum printString = 'Lg.Lw.Ly.{<y> + (<g> <~ <w>)}'.

	specializedSum _ sum <~ (Lambda n * 10) .
	self assert: specializedSum printString = 'Lw.Ly.{<y> + (<w> * 10)}'.
	self assert: (specializedSum <~~ #(5 1)) = 51.

	wronglyExpandedSpecializedSum _ sum <= (Lambda n * 10) .
	self assert: wronglyExpandedSpecializedSum printString
		= 'Ln.Lw.Ly.{<y> + ((<n> * 10) <~ <w>)}' .
	self deny: (wronglyExpandedSpecializedSum <~~ #(5 1)) = 51.

	expandedSpecializedSum _ sum <@ (Lambda n * 10) .
	self assert: expandedSpecializedSum printString 
		= 'Lw.Ly.{<y> + (Ln.{<n> * 10} <~ <w>)}'.
	self assert: (expandedSpecializedSum <~~ #(5 1)) = 51.

! !

!FunctionalTalkTest methodsFor: 'lambda calculus' stamp: 'spfa 2/9/2007 15:50'!
testHighOrder3

	| o f g |

	o _ Lambda o.
	f _ (Lambda x default: 1) + 4.
	g _ f lambda quoted parametrize: #x default: o.

	self assert: (f defaultFor: #x) = 1.
	g <~ 18.
	self assert: (f defaultFor: #x) = 18.
	g <~ (5 lambda + 4).
	self assert: (f defaultFor: #x) printString = '(5 + 4)' ! !

!FunctionalTalkTest methodsFor: 'lambda calculus' stamp: 'spfa 2/9/2007 16:00'!
testHighOrder4

	| s f g |

	s _ LambdaSlot id: #s .
	s default: 10.

	self assert: s default = 10.

	f _ s quoted default.
	self assert: f value = 10.

	s default: 5 lambda + 1.
	self assert: s default printString = '(5 + 1)'.
	self assert: f value printString = '(5 + 1)'.

	g _ s quoted default quoted lambdaValue.
	self assert: g value = 6.

! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 1/19/2007 19:47'!
testInliningVsSubstitution

	| x y |

	x _ Lambda x.
	y _ Lambda y.

	self assert: ((x do: 1) <~ y) printString = '(Ly.{<y>} do: 1)'.
	self assert: ((x do: 1) <@ y) printString = '(Ly.{<y>} do: 1)'.
	self assert: ((x do: 1) <= y) printString = 'Ly.{<y> do: 1}'.

	self assert: ((1+x) <~ y) printString = '(1 + Ly.{<y>})'.
	self assert: ((1+x) <@ y) printString = '(1 + Ly.{<y>})'.
	self assert: ((1+x) <= y) printString = 'Ly.{1 + <y>}'.! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 2/9/2007 21:38'!
testLambdaVsQuoted

	"
	#lambda and #quoted can both be sent to any object. 

	#lambda returns a lambda expression which evaluates to its receiver
	#quoted forces its receiver to form a lambda expression with the following message

	although they may seem similar, these are completely different operations !!
	their usages differ accordingly:

	#lambda references a function in an higher-order expression
	#quoted inhibits reduction/evaluation during the construction of an expression (it allows seamless extension)
"

	 {
		'string' . 
		Class . 
		Morph new . 
		LambdaSlot id: #n .
		Lambda x + 1.
		#hey .
		self .
		18
	}
	do: [:o |
		self assert: o lambda isMessageSend.
		self assert: o lambda value == o.

		self assert: o quoted value isMessageSend.
		self assert: o quoted isMessageSend isMessageSend.
	].
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 2/9/2007 20:13'!
testLMSVsBlock

	| b f fx fy c g sum |

	"let's compare two ways to define a function of two variables"

	"with a block ..."
	b _ [:x :y | ((x*x) + (y*y)) sqrt] fixTemps.

	"... with a lambda expression"
	fx _ Lambda x.                 
	fy _ Lambda y.
	f _ ((fx*fx) + (fy*fy)) sqrt.

	"both perform the same calculation"
	self assert: (b value: 3 value: 4) = (f <~~ {3 . 4}).

	"we can in both cases reduce the number of arguments (currying)"
	c _ [:x | b value: x value: 0] fixTemps.
	g _ f <<~ (#y -> 0).
	self assert: (c value: 4) = (g <~ 4).

	"lambda function printing is transparent"
	self assert: c printString = 
		'[] in FunctionalTalkTest>>testLMSVsBlock {[:x | b value: x value: 0]}'.
	self assert: g printString = 'Lx.{((<x> * <x>) + 0) sqrt}'.

	"adding blocks is cumbersome since arguments must be explicited"
	self should: [b + c] raise: Error.
	sum _ [:x :y | (b value: x value: y) + (c value: x)] fixTemps.
	self assert: (sum value: 3 value: 4) = 8.

	"lambda expressions are much easier to handle"
	self assert: (f + g) isLambda.
	self assert: (f + g) <~~ {3 . 4} = 8.

	self assert: (f + (2 * g log)) isLambda.
	self assert: ((f + (2 * g log)) <~~ {3 . 4} closeTo: 5.954242509439325).

	"... but we should care about the sharing of variables:
	here by construction g and f share the x slot:"
	self assert: (f + g) printString 
		= 'Lx.Ly.{((<x> * <x>) + (<y> * <y>)) sqrt + ((<x> * <x>) + 0) sqrt}'.
	self assert: (f <~~ {3 . 4}) + (g <~ 3) = ((f +g) <~~ {3 . 4}).
	"by lambdaCopy-ing we can force the x slots to become independent"
	self assert: (f lambdaCopy + g) printString 
		= 'Lx1.Ly.Lx.{((<x1> * <x1>) + (<y> * <y>)) sqrt + ((<x> * <x>) + 0) sqrt}'.
	self assert: (f <~~ {3 . 4}) + (g <~ 4) = ((f lambdaCopy + g) <~~ {3 . 4 . 4}).

	"note that a lambda expression is internally compiled as a block for efficiency" 
	self assert: f compiled isBlock.
	self assert: (f compiled value: 3 value: 4) = 5.
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 2/9/2007 20:02'!
testLMSVsBlock2

	"as far as local variables are concerned, lambda expressions behave the same as a block with #fixTemps"

	| b c f fx fy a col |

	a _ 10.

	b _ [:x :y | a + ((x*x) + (y*y)) sqrt].
	c _ [:x :y | a + ((x*x) + (y*y)) sqrt] fixTemps.

	fx _ Lambda x.
	fy _ Lambda y.
	f _ a + ((fx*fx) + (fy*fy)) sqrt.

	self assert: (b value: 3 value: 4) = (f <~~ {3 . 4}).
	self assert: (c value: 3 value: 4) = (f <~~ {3 . 4}).
	a _ 0.
	self deny: (b value: 3 value: 4) = (f <~~ {3 . 4}).
	self assert: (c value: 3 value: 4) = (f <~~ {3 . 4}).


	"another example"

	col := (1 to: 10) collect: [:each | [each]].
	self assert: col first value = 10. 

	col := (1 to: 10) collect: [:each | [each] fixTemps].
	self assert: col first value = 1. 

	col := (1 to: 10) collect: [:each | each lambda].
	self assert: col first value = 1. 
	
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 2/9/2007 20:13'!
testLMSVsBlock3

	| b c d f g h |

	"here is a substitution example rather cumbersome with blocks..."

	b _ [:x :y | x + y] fixTemps.
	c _ [:z | z sqrt] fixTemps.

	d _ [:m :n | b value: (c value: m) value: n] fixTemps.

	self assert: (d value: 4 value: 1) = 3.
	self assert: d printString = '[] in FunctionalTalkTest>>testLMSVsBlock3 {[:m :n | b   value: (c value: m)   value: n]}'.


	"... and much clearer with lambda expressions:"

	f _ Lambda x + Lambda y.
	g _ Lambda z sqrt.

	h _ g => f.

	self assert: #(4 1) ~~> h = 3.
	self assert: h printString = 'Lz.Ly.{<z> sqrt + <y>}'.


! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 14:07'!
testObfuscated

	| p mp |

	p _ Point lambda: #x:y: .
	p _ p <== {1 lambda: #+ slotIds: {1} . 2 lambda: #* slotIds: {2}}. 

	self assert: p printString = 'L1.L2.{Point x: (1 + <1>) y: (2 * <2>)}'.
	self assert: {5 . 15} ~~> p = (6 @ 30).

	mp _ p <== {5 .  (Point lambda: #x:y: slotIds: {'x2' . 'y2'}) r}.
	self assert: mp printString = 'Lx2.Ly2.{Point x: 6 y: (2 * (Point x: <x2> y: <y2>) r)}'.
	self assert: {0 . 1} ~~> mp = (6 @ 2)
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 2/2/2007 20:44'!
testParameters

	| p q pq |

	"as a convenience, arbitrary slots can be defined as parameters.

parameters have a default value, and sending #default to a lambda function returns a reduced copy of this function where all parameter slots have been substituted with their default values.

a parameter can also have a description string"

	p _ (Point lambda: #x:y:) + (Lambda point default: (0@0); description: 'origin').
	self assert: p isParametric.
	self assert: p default printString = ((Point lambda: #x:y:) + (0@0)) printString.
	self assert: p parameters size = 1.
	self assert: p parameters first id = #point.
	self assert: p parameters first default = (0@0).
	self assert: p parameters first description = 'origin'.

	"changing the default value for a parameter"
	p parametrize: #point default: (-1@1).
	self assert: p default printString = ((Point lambda: #x:y:) + (-1@1)) printString.

	q _ Lambda x + Lambda y.
	self deny: q isParametric. 
	self assert: q default printString = q printString.
	self assert: {5. 10} ~~> q = 15.

	"making a parametric version of q"
	pq _ {#x -> (Lambda x default: 5) . #y -> (Lambda y default: 10)} ==>> q.
	self assert: pq isParametric. 
	self assert: pq default = 15.
 	"sending #defaultForm returns a copy with parameters substituted, without reduction:"
	self assert: pq defaultForm printString = '(5 + 10)'.

	"having q itself become parametric"
	q parametrize: #x default: 5.
	q parametrize: #y default: 10.
	self assert: q isParametric. 
	self assert: q default = 15.	
	self assert: q defaultForm printString = '(5 + 10)'.
	"do not overuse this because lambda expressions are not supposed to be mutable !!"


	
	! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/4/2006 14:58'!
testPrinting1

	| f g |

	f _ LambdaSlot selector: #+ arguments: {56}.
	g _ ((f sqrt) + 2) * f. 

	self assert: g printString 
		= 'Lreceiver.{((<receiver> + 56) sqrt + 2) * (<receiver> + 56)}' 

! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/4/2006 14:59'!
testPrinting2

	| f g f2 ff |

	f _ 56 lambda: #+ .
	g _ f sqrt.
	f2 _ f * 2.
	ff _ f * f. 

	self assert: f printString = 'L1.{56 + <1>}'.
	self assert: g printString = 'L1.{(56 + <1>) sqrt}'.
	self assert: f2 printString = 'L1.{(56 + <1>) * 2}'.
	self assert: ff printString = 'L1.{(56 + <1>) * (56 + <1>)}'

! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 14:08'!
testPrinting3

	| p sp |

	p _ Point lambda: #x:y: .
	sp _ p <== {0 . Lambda y}.

	self assert: p printString = 'L1.L2.{Point x: <1> y: <2>}'.
	self assert: sp printString = 'Ly.{Point x: 0 y: <y>}'.
	p <~~ {1 . 2}.
	self assert: p printString = 'L1.L2.{Point x: <1> y: <2>}'.
	sp <~ 10.
	self assert: sp printString = 'Ly.{Point x: 0 y: <y>}'.
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 2/9/2007 20:06'!
testQuoted

	| f g h |

	"#quoted must be used instead of #lambda before any selector belonging to the LambdaMessageSend protocol. 
	it prevents the lambda expression itself to immediately answer the message; instead it allows the selector to be included in the lambda.

	see #testLambdaVsQuoted for more about #lambda and #quoted"

	f _ 4 lambda = (2 lambda + 2).
	self assert: (f basicType == #Boolean). 
	self deny: f.
	g _ 4 quoted = (2 lambda + 2).
	self assert: g isMessageSend.
	self assert: g printString = '(4 = (2 + 2))'.
	self assert: g value.
	h _ 4 quoted = (2 quoted + 2).
	self assert: h isMessageSend.
	self assert: h printString = '(4 = (2 + 2))'.
	self assert: h value.
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 1/19/2007 19:21'!
testRenameSlots

	| f |

	f _ Behavior lambda: #read:go:buy:cook:inviteForDinner:.
	f slotIds: #(recipe shop food dish girlfriend).

	self assert: f slotIds = #(recipe shop food dish girlfriend).

	self assert: (f <<~ (#girlfriend -> #bob)) slotIds = #(recipe shop food dish).! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 15:35'!
testSection

	| p sp a sa ssa sssa |

	p _ Point lambda: #x:y:.
	sp _ p <~~ {5 . Lambda}.

	self assert: (sp <~ 3) = (5@3).

	a _ Array lambda: #with:with:with:with:with:with:.
	sa _ a <~~ {Lambda . Lambda. Lambda . #hello . Lambda . Lambda}.
	ssa _ sa <~~ {Lambda . #two . Lambda . #five . Lambda}.
	sssa _ ssa <~~ {#a . #b . Lambda}.

	self assert: (sa boundSlots size = 5).
	self assert: (sa <~~ {1 . 2 . 3 . 4 . 5}) = {1 . 2 . 3 . #hello . 4 . 5}.

	self assert: (ssa boundSlots size = 3).
	self assert: (ssa <~~ {1 . 2 . 3}) = {1 . #two . 2 . #hello . #five . 3}.

	self assert: (sssa boundSlots size = 1).
	self assert: (sssa <~ 1) = {#a . #two . #b . #hello . #five . 1}.


	
	! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/8/2007 14:17'!
testSerialization

	| f rs g |

	f _ Lambda x + (Time quoted primSecondsClock).
	f compile. 
	self assert: f hasCompiledForm.

	rs _ SmartRefStream streamedRepresentationOf: f.

	self deny: f hasCompiledForm.

	g _ SmartRefStream objectFromStreamedRepresentation: rs.

	self deny: g hasCompiledForm.
	self assert: f printString = g printString.

	f _ BlobMorph new quoted openInWorld.
	f compile.
	self assert: f hasCompiledForm.

	rs _ SmartRefStream streamedRepresentationOf: f.

	self deny: f hasCompiledForm.

	g _ SmartRefStream objectFromStreamedRepresentation: rs.

	self deny: g hasCompiledForm.
	self deny: f printString = g printString.! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 15:35'!
testSerialization2

	| x f g rs a |

	x _ Lambda x.
	f _ 2+x.
	g _ x * x.

	rs _ SmartRefStream streamedRepresentationOf: {f . g}.
	a _ SmartRefStream objectFromStreamedRepresentation: rs.

	self assert: (a first + a second) boundSlots size = 1.
	self assert: (3 ~> (a first + a second)) = 14
! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 1/19/2007 13:13'!
testShortAPI

	| f g |

	"all arrow notations work in both direction, although parenthesis are needed in a chain from left to right when pointing to a non-lambda object"

	f _ Lambda x - Lambda y.

	self assert: (f <~~ {3 . 4}) = -1.
	self assert: ({3 . 4} ~~> f) = -1.

	self assert: (f <~ 3) printString = 'Ly.{3 - <y>}'.
	self assert: (3 ~> f) printString = 'Ly.{3 - <y>}'.

	self assert: (f <~ 3 <~ 4) = -1.
	self assert: (4 ~> (3 ~> f)) = -1.
	self should: [4 ~> 3 ~> f] raise: Error.

	self assert: (f <~~ {Lambda . 4}) printString = 'Lx.{<x> - 4}' .
	self assert: ({Lambda . 4} ~~> f) printString = 'Lx.{<x> - 4}' .

	self assert: (f <~~ {Lambda . 4} <~ 3) = -1.
	self assert: (3~> ({Lambda . 4} ~~> f)) = -1.

	self assert: (f <<~ (#y -> 4)) printString = 'Lx.{<x> - 4}'.
	self assert: ((#y -> 4) ~>> f) printString = 'Lx.{<x> - 4}'.

	self assert: (f <<~~ {#y -> 4 . #x -> 3}) = -1.
	self assert: ({#y -> 4 . #x -> 3} ~~>> f) = -1.

	self assert: (f <@@ {3 . 4}) printString = '(3 - 4)'.
	self assert: ({3 . 4} @@> f) printString = '(3 - 4)'.

	self assert: (f <@ 3) printString = 'Ly.{3 - <y>}'.
	self assert: (3 @> f) printString = 'Ly.{3 - <y>}'.
	
	self assert: (f <@ 3 <@ 4) printString = '(3 - 4)'.
	self assert: (4 @> (3 @> f)) printString = '(3 - 4)'.
	self should: [4 @> 3 @> f] raise: Error.

	self assert: (f <// {3 . 4}) = -1.
	self assert: ({3 . 4} //> f) = -1.
	self should: [f </ 3] raise: Error.

	g _ (1 + Lambda x) * (2 - Lambda y).
	self assert: (g <<= (#y -> 2)) printString = 'Lx.{(1 + <x>) * 0}'.
	self assert: (g <<== {#y -> 2 . #x -> 1}) printString = '0'.
	self assert: ((#x -> (Lambda z - 1)) =>> g) 
						printString = 'Lz.Ly.{(1 + (<z> - 1)) * (2 - <y>)}' .
	self assert: ({#y -> 2} ==>> g) printString = 'Lx.{(1 + <x>) * 0}' 
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/9/2007 22:02'!
testSlotCopying

	| x t u f |

	x _ Lambda x.
	t _ Lambda t.
	u _ Lambda u.
	f _ x quoted <~ t.

	self deny: (f <@ t ) printString = 'Lt.{<t> <~ <t>}'.      
	self assert: (f <@ t lambdaCopy) printString = 'Lt.{Lt1.{<t1>} <~ <t>}'.  
	self assert: (f <@ u) printString = 'Lt.{Lu.{<u>} <~ <t>}'.  

	self assert: (f <@ t) printString = 'Lt.{Lt1.{<t1>} <~ <t>}'.  
	self assert: (f <~ t) printString = 'Lt.{Lt1.{<t1>} <~ <t>}'.  
	self assert: (f <= t) printString = 'Lt1.Lt.{<t1> <~ <t>}'.  

	self assert: (f <== {u . u}) printString = 'Lu.{<u> <~ <u>}'.  ! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/10/2007 11:30'!
testSlotCopying2

	| x y f u |

	x _ Lambda x.
	y _ Lambda y.
	f _ x + y.
	u _ Lambda u.

	self assert: (f <== {u . u}) printString = 'Lu.{<u> + <u>}'.  
	self assert: (f <== {x . x}) printString = 'Lx.{<x> + <x>}'.  
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 12/11/2006 15:34'!
testSubstituteSlot

	| p sp a sa ssa sssa |

	p _ Point lambda: #x:y: slotIds: #(x y).
	sp _ p <<~ (#x -> 5).

	self assert: (sp <~ 3) = (5@3).

	a _ Array lambda: #with:with:with:with:with:with:.
	sa _ a <<~ (#'4' -> #hello).
	ssa _ sa <<~~ {#'2' -> #two . #'5' -> #five}.
	sssa _ ssa <<~~ {#'1' -> #a . #'3' -> #b}.

	self assert: (sa boundSlots size = 5).
	self assert: (sa <~~ {1 . 2 . 3 . 4 . 5}) = {1 . 2 . 3 . #hello . 4 . 5}.

	self assert: (ssa boundSlots size = 3).
	self assert: (ssa <~~ {1 . 2 . 3}) = {1 . #two . 2 . #hello . #five . 3}.

	self assert: (sssa boundSlots size = 1).
	self assert: (sssa <~ 1) = {#a . #two . #b . #hello . #five . 1}.


	
	! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 1/19/2007 19:22'!
testSubstituteSlot2

	| f sf ssf sssf |

	f _ Behavior lambda: #read:go:buy:cook:inviteForDinner: 
		slotIds: #(recipe shop food dish girlfriend).

	sf _ (#recipe -> (Preferences lambda: #cookbook: slotId: #book)) =>> f.

	ssf _ {
			#shop -> (Utilities lambda: #store:inCity: slotIds: {#store . #city}).
			#girlfriend -> (Preferences lambda: #girlNamed: slotId: #lovedOne)
		}==>> sf.

	sssf _ {
				#dish -> #couscous .
				 #food -> (Utilities lambda: #allYouNeedForAGoodCouscous)
		}==>> ssf.

	self assert: 
	sf slotIds = #(book shop food dish girlfriend).
	self assert: 
	ssf slotIds = #(book store city food dish lovedOne). .
	self assert: 
	sssf slotIds = #(book store city lovedOne).

	self assert: 
	sssf printString = 'Lbook.Lstore.Lcity.LlovedOne.{Behavior read: (Preferences cookbook: <book>) go: (Utilities store: <store> inCity: <city>) buy: Utilities allYouNeedForAGoodCouscous cook: #couscous inviteForDinner: (Preferences girlNamed: <lovedOne>)}' 
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/8/2007 22:05'!
testSubstituteSlot3

	| f |

	f _ Lambda point r.

	self assert: (f valueWithArguments: {3@4}) = 5.
	self assert: (f <~ (3@4)) = 5.
	self assert: (f <= (3@4)) = 5.
	self assert: (f <@ (3@4)) printString = '(3@4) r'.

	self assert: (f valueWithArguments: {Point lambda x: 3 y: 4}) printString = '(Point x: 3 y: 4) r'.

	self assert: (f <~ (Point lambda x: 3 y: 4)) printString = '(Point x: 3 y: 4) r'.
	self assert: (f <= (Point lambda x: 3 y: 4)) printString = '(Point x: 3 y: 4) r'.
	self assert: (f <@ (Point lambda x: 3 y: 4)) printString = '(Point x: 3 y: 4) r'.! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/8/2007 13:25'!
testVariables

	| f x y |

	x _ Lambda x.

	self assert: x boundVariables = #(x).
	self assert: x <~ 5 = 5.

	y _ Lambda y.
	f _ x ofVariables: {x . y}.

	self assert: f boundVariables = #(x y).
	self deny: f <~ 5 = 5.
	self assert: f <~ 5 <~ #anything = 5.

	f _ x ofVariables: #().

	self assert: f boundVariables = #().
	self should: [f <~ 5] raise: Error
	! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/8/2007 13:26'!
testVariables2

	| f g x y |

	x _ Lambda x.
	f _ x + 1.

	self assert: f boundVariables = #(x).
	self assert: f <~ 5 = 6.

	y _ Lambda y.
	g _ f ofVariables: {x . y}.
	
	self assert: g boundVariables = #(x y).
	self deny: g <~ 5 = 6.
	self assert: g <~ 5 <~ #anything = 6.

	g _ f ofVariables: #().

	self assert: g boundVariables = #().
	self should: [g <~ 5] raise: Error! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/8/2007 13:35'!
testVariables3

	| s f |

	s _ Lambda s.
	f _ Lambda f sister age.

	self assert: ((s ofVariables: {s . Lambda dummy}) <= f) printString =
 		 'Lf.Ldummy.{<f> sister age}'.
	self assert: ((s ofVariables: {s . Lambda dummy}) <@ f) printString =
 		 'Ldummy.{Lf.{<f> sister age}}'.
	self assert: ((s ofVariables: {s . Lambda dummy}) <~ f) printString =
 		 'Ldummy.{Lf.{<f> sister age}}'.
! !

!FunctionalTalkTest methodsFor: 'tests' stamp: 'spfa 2/9/2007 19:55'!
testVariations

	{
		Character lambda codePoint: Lambda c.
		Character lambda: #codePoint: with: Lambda c.
 		Character quoted codePoint: Lambda c
	}
	do: [:f |.
		self assert: f printString =  'Lc.{Character codePoint: <c>}'.
		self assert: (f <~ 97) == $a .  
		self assert: (f <@ 97) printString = '(Character codePoint: 97)'. 
	]! !

!FunctionalTalkTest methodsFor: 'documentation' stamp: 'spfa 2/9/2007 19:55'!
testWithLambdaArity

	"#withLambdaArity: can be used to construct a lambda expression taking an arbitrary number of arguments and evaluating to the receiver"

	self assert: (5 withLambdaArity: 1) <~ 'beuh' = 5.
	self assert: (5 withLambdaArity: 3) <~~ #(8 'beuh' yo) = 5.
	self assert: ((Lambda x + 1) withLambdaArity: 2) <~~ #(ga zo) <~ 5 = 6.! !

!LambdaMessageSend methodsFor: 'atomic access' stamp: 'spfa 2/5/2007 09:43'!
type

	self isAtomic ifFalse: [self error: 'not an atomic lambda'].

	^ receiver type! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 2/5/2007 09:33'!
type

	^ type! !

!LambdaMessageSend methodsFor: 'atomic access' stamp: 'spfa 2/5/2007 09:43'!
type: anObject

	self isAtomic ifFalse: [self error: 'not an atomic lambda'].

	receiver type: anObject! !

!LambdaSlot methodsFor: 'accessing' stamp: 'spfa 2/5/2007 09:33'!
type: aClass

	type _ aClass! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 1/19/2007 19:10'!
unboundSlotIds

	^ self unboundSlots collect: [:uslot | uslot id].! !

!LambdaMessageSend methodsFor: 'accessing' stamp: 'spfa 1/19/2007 16:15'!
unboundSlots

	^ self getSlots reject: [:s | self boundSlots identityIncludes: s]! !

!LambdaMessageSend methodsFor: 'parameters' stamp: 'spfa 2/4/2007 18:22'!
update: anything

	"if the receiver is the dependent of some object (for example one of its parameters), and that object changes, then inform the receiver's own dependents"

	self changed! !

!LambdaMessageSend methodsFor: 'evaluating' stamp: 'spfa 2/8/2007 22:58'!
value

	^ self valueRecursive: true
! !

!LambdaMessageSend methodsFor: 'numeric util.' stamp: 'spfa 2/5/2007 09:07'!
valueAt: arg

	^ self valueAt: arg ifUndefined: nil! !

!LambdaMessageSend methodsFor: 'numeric util.' stamp: 'spfa 2/5/2007 09:07'!
valueAt: arg ifUndefined: aNumber

	"we assume this function is supposed to evaluate to numeric values:
return aNumber if the function is not defined at arg"

	| v |

	v _ [self compiled value: arg] on: Error do: [^ aNumber].
	(v isNaN | v isInfinite) ifTrue: [^ aNumber].

	^ v! !

!LambdaMessageSend methodsFor: 'evaluating' stamp: 'spfa 2/9/2007 15:02'!
valueRecursive: flag

	arguments isLambda ifTrue: [^ self].

	self boundSlots ifNotEmpty: [^ self].

	selector ifNil: 
		[^ flag ifTrue: [receiver lambdaValue] ifFalse: [receiver]].

	selector isLambda ifTrue: [^ self].

	^ [self forcedValueRecursive: flag] on: Error do: [self]
! !

!LambdaMessageSend methodsFor: 'numeric util.' stamp: 'spfa 2/5/2007 09:08'!
valuesOn: aCollection

	^ aCollection collect: [:x | self valueAt: x ifUndefined: nil]! !

!LambdaMessageSend methodsFor: 'numeric util.' stamp: 'spfa 2/5/2007 09:07'!
valuesOn: aCollection ifUndefined: aNumber

	^ aCollection collect: [:x | self valueAt: x ifUndefined: aNumber]! !

!LambdaMessageSend methodsFor: 'evaluating' stamp: 'spfa 2/2/2007 20:44'!
valueWithArguments: anArray

	^ self <~~ anArray! !

!IdentityDictionary class methodsFor: '*FunctionalTalk' stamp: 'spfa 12/8/2006 14:59'!
withAssociations: anArray

	anArray class == self ifTrue: [^ anArray].
	^ self newFrom: anArray! !

!Object methodsFor: '*FunctionalTalk' stamp: 'spfa 2/6/2007 13:38'!
withLambdaArity: aNumber

	"idiomatic - returns a lambda expression taking aNumber of arguments and returning self"

	| s slots |
	s _ Lambda s.

	slots _ Array streamContents: [:str |
		str nextPut: s.
		(1 to: aNumber) do: [:n | 
			str nextPut: (Lambda perform: ('a', n asString) asSymbol)]].

	^ (s ofVariables: slots) <~ self  ! !

!LambdaMessageSend methodsFor: 'conversion' stamp: 'spfa 12/10/2006 23:09'!
withNewSlots: aDictionary

	^ self copy
		receiver: (receiver withNewSlots: aDictionary);
		selector: (selector withNewSlots: aDictionary);
		arguments: (arguments isLambda 
						ifTrue: [arguments withNewSlots: aDictionary]
						ifFalse: [arguments collect: [:a | a withNewSlots: aDictionary]]);
		slots: (self boundSlots collect: [:bs | aDictionary at: bs])
		! !

!LambdaSlot methodsFor: 'conversion' stamp: 'spfa 12/10/2006 22:52'!
withNewSlots: aDictionary

	^ aDictionary at: self! !

!Object methodsFor: '*FunctionalTalk' stamp: 'spfa 12/10/2006 22:51'!
withNewSlots: aDictionary

	^ self! !

!LambdaMessageSend methodsFor: 'printing' stamp: 'spfa 1/19/2007 21:58'!
writeConstructorOn: stream 

	stream nextPutAll: '([| '.
	self boundSlots do: [:s | s lambdaStoreOn: stream. stream space].
	stream nextPutAll: '| '.
	self boundSlots do: [:s | s declareOn: stream].
	self lambdaStoreOn: stream.
	stream nextPutAll: '] value)'! !

!SequenceableCollection methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/11/2006 13:28'!
//> aLambdaMessageSend

	^ aLambdaMessageSend reduceWithAllNonLambda: self! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/11/2006 13:28'!
/> aLambdaMessageSend

	^ aLambdaMessageSend reduceWithNonLambda: self! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 5/11/2006 10:18'!
</ arg

	^ self reduceWithNonLambda: arg

! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 5/11/2006 10:18'!
<// arg

	^ self reduceWithAllNonLambda: arg

! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 12/11/2006 13:50'!
<<= anAssociation

	^ self inlineSlot: anAssociation 


! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 12/11/2006 13:50'!
<<== associations

	^ self inlineSlots: associations 


! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 5/23/2006 14:23'!
<<@ anAssociation

	^ self substituteSlot: anAssociation

	! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 12/8/2006 15:13'!
<<@@ associations

	^ self substituteSlots: associations

	! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 5/10/2006 15:33'!
<<~ anAssociation

	^ self reduceWithSlot: anAssociation 


! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 12/8/2006 15:15'!
<<~~ associations

	^ self reduceWithSlots: associations


! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 12/11/2006 13:56'!
<= anAssociation

	^ self inlineSlots: (self argumentsFromList: {anAssociation}) 


! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 12/11/2006 14:14'!
<== associations

	^ self inlineSlots: (self argumentsFromList: associations) 


! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 5/10/2006 15:32'!
<@ anObject

	^ self substitute: {anObject}

	! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 5/10/2006 15:32'!
<@@ anArray

	^ self substitute: anArray

	! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 5/9/2006 14:19'!
<~ arg

	^ self reduceWith: arg 

! !

!LambdaMessageSend methodsFor: 'short API' stamp: 'spfa 5/9/2006 14:18'!
<~~ args

	^ self reduceWithAll: args

! !

!LambdaSlot methodsFor: 'testing' stamp: 'spfa 5/14/2006 14:04'!
= anObject

	^ anObject isLambdaSlot! !

!SequenceableCollection methodsFor: '*FunctionalTalk-API' stamp: 'spfa 12/11/2006 13:58'!
==> aLambdaMessageSend

	^ aLambdaMessageSend inlineSlots: (aLambdaMessageSend argumentsFromList: self)! !

!SequenceableCollection methodsFor: '*FunctionalTalk-API' stamp: 'spfa 12/11/2006 13:52'!
==>> aLambdaMessageSend

	^ aLambdaMessageSend inlineSlots: self! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 12/11/2006 13:53'!
=> aLambdaMessageSend

	^ aLambdaMessageSend inlineSlots: (aLambdaMessageSend argumentsFromList: {self})! !

!Association methodsFor: '*FunctionalTalk-API' stamp: 'spfa 12/11/2006 13:52'!
=>> aLambdaMessageSend

	^ aLambdaMessageSend inlineSlot: self! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/11/2006 13:26'!
@> aLambdaMessageSend

	^ aLambdaMessageSend substitute: {self}! !

!Association methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/23/2006 14:24'!
@>> aLambdaMessageSend

	^ aLambdaMessageSend substituteSlot: self! !

!SequenceableCollection methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/11/2006 13:26'!
@@> aLambdaMessageSend

	^ aLambdaMessageSend substitute: self! !

!SequenceableCollection methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/23/2006 14:24'!
@@>> aLambdaMessageSend

	^ aLambdaMessageSend substituteSlots: self! !

!Object methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/11/2006 13:20'!
~> aLambdaMessageSend

	^ aLambdaMessageSend reduceWith: self! !

!Association methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/11/2006 13:31'!
~>> aLambdaMessageSend

	^ aLambdaMessageSend reduceWithSlot: self! !

!SequenceableCollection methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/11/2006 13:26'!
~~> aLambdaMessageSend

	^ aLambdaMessageSend reduceWithAll: self! !

!SequenceableCollection methodsFor: '*FunctionalTalk-API' stamp: 'spfa 5/11/2006 13:30'!
~~>> aLambdaMessageSend

	^ aLambdaMessageSend reduceWithSlots: self! !
